import pytest
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Question, QuestionSet
from ..serializers.v1 import QuestionSerializer
from ..validators import QuestionLockedValidator


def test_create(db):
    QuestionLockedValidator()({
        'questionset': QuestionSet.objects.first(),
        'locked': False
    })


def test_create_locked(db):
    QuestionLockedValidator()({
        'questionset': QuestionSet.objects.first(),
        'locked': True
    })


def test_update(db):
    question = Question.objects.first()

    QuestionLockedValidator(question)({
        'questionset': question.questionset,
        'locked': False
    })


def test_update_error(db):
    question = Question.objects.first()
    question.locked = True
    question.save()

    with pytest.raises(ValidationError):
        QuestionLockedValidator(question)({
            'questionset': question.questionset,
            'locked': True
        })


def test_update_parent_error(db):
    question = Question.objects.first()
    question.questionset.locked = True
    question.questionset.save()

    with pytest.raises(ValidationError):
        QuestionLockedValidator(question)({
            'questionset': question.questionset,
            'locked': False
        })


def test_update_parent_parent_error(db):
    question = Question.objects.first()
    question.questionset.section.locked = True
    question.questionset.section.save()

    with pytest.raises(ValidationError):
        QuestionLockedValidator(question)({
            'questionset': question.questionset,
            'locked': False
        })


def test_update_parent_parent_parent_error(db):
    question = Question.objects.first()
    question.questionset.section.catalog.locked = True
    question.questionset.section.catalog.save()

    with pytest.raises(ValidationError):
        QuestionLockedValidator(question)({
            'questionset': question.questionset,
            'locked': False
        })


def test_update_lock(db):
    question = Question.objects.first()

    QuestionLockedValidator(question)({
        'questionset': question.questionset,
        'locked': True
    })


def test_update_unlock(db):
    question = Question.objects.first()
    question.locked = True
    question.save()

    QuestionLockedValidator(question)({
        'questionset': question.questionset,
        'locked': False
    })


def test_serializer_create(db):
    validator = QuestionLockedValidator()
    validator.set_context(QuestionSerializer())

    validator({
        'questionset': QuestionSet.objects.first(),
        'locked': False
    })


def test_serializer_create_locked(db):
    validator = QuestionLockedValidator()
    validator.set_context(QuestionSerializer())

    validator({
        'questionset': QuestionSet.objects.first(),
        'locked': True
    })


def test_serializer_update(db):
    question = Question.objects.first()

    validator = QuestionLockedValidator()
    validator.set_context(QuestionSerializer(instance=question))

    validator({
        'questionset': question.questionset,
        'locked': False
    })


def test_serializer_update_error(db):
    question = Question.objects.first()
    question.locked = True
    question.save()

    validator = QuestionLockedValidator()
    validator.set_context(QuestionSerializer(instance=question))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'questionset': question.questionset,
            'locked': True
        })


def test_serializer_update_parent_error(db):
    question = Question.objects.first()
    question.questionset.locked = True
    question.questionset.save()

    validator = QuestionLockedValidator()
    validator.set_context(QuestionSerializer(instance=question))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'questionset': question.questionset,
            'locked': True
        })


def test_serializer_update_parent_parent_error(db):
    question = Question.objects.first()
    question.questionset.section.locked = True
    question.questionset.section.save()

    validator = QuestionLockedValidator()
    validator.set_context(QuestionSerializer(instance=question))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'questionset': question.questionset,
            'locked': True
        })


def test_serializer_update_parent_parent_parent_error(db):
    question = Question.objects.first()
    question.questionset.section.catalog.locked = True
    question.questionset.section.catalog.save()

    validator = QuestionLockedValidator()
    validator.set_context(QuestionSerializer(instance=question))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'questionset': question.questionset,
            'locked': True
        })


def test_serializer_update_lock(db):
    question = Question.objects.first()

    validator = QuestionLockedValidator()
    validator.set_context(QuestionSerializer(instance=question))

    validator({
        'questionset': question.questionset,
        'locked': True
    })


def test_serializer_update_unlock(db):
    question = Question.objects.first()
    question.locked = True
    question.save()

    validator = QuestionLockedValidator()
    validator.set_context(QuestionSerializer(instance=question))

    validator({
        'questionset': question.questionset,
        'locked': False
    })



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-28 12:34
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0012_meta'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='catalog',
            options={'ordering': ('order',), 'permissions': (('view_catalog', 'Can view Catalog'),), 'verbose_name': 'Catalog', 'verbose_name_plural': 'Catalogs'},
        ),
        migrations.AlterModelOptions(
            name='question',
            options={'permissions': (('view_question', 'Can view Question'),), 'verbose_name': 'Question', 'verbose_name_plural': 'Questions'},
        ),
        migrations.AlterModelOptions(
            name='questionentity',
            options={'ordering': ('subsection__section__catalog__order', 'subsection__section__order', 'subsection__order', 'order'), 'permissions': (('view_questionentity', 'Can view Question entity'),), 'verbose_name': 'Question entity', 'verbose_name_plural': 'Question entities'},
        ),
        migrations.AlterModelOptions(
            name='section',
            options={'ordering': ('catalog__order', 'order'), 'permissions': (('view_section', 'Can view Section'),), 'verbose_name': 'Section', 'verbose_name_plural': 'Sections'},
        ),
        migrations.AlterModelOptions(
            name='subsection',
            options={'ordering': ('section__catalog__order', 'section__order', 'order'), 'permissions': (('view_subsection', 'Can view Subsection'),), 'verbose_name': 'Subsection', 'verbose_name_plural': 'Subsections'},
        ),
    ]



from django.db import models

from rdmo.core.managers import (AvailabilityManagerMixin,
                                AvailabilityQuerySetMixin,
                                CurrentSiteManagerMixin,
                                CurrentSiteQuerySetMixin, GroupsManagerMixin,
                                GroupsQuerySetMixin)


class ViewQuestionSet(CurrentSiteQuerySetMixin, GroupsQuerySetMixin, AvailabilityQuerySetMixin, models.QuerySet):

    def filter_catalog(self, catalog):
        return self.filter(models.Q(catalogs=None) | models.Q(catalogs=catalog))


class ViewManager(CurrentSiteManagerMixin, GroupsManagerMixin, AvailabilityManagerMixin, models.Manager):

    def get_queryset(self):
        return ViewQuestionSet(self.model, using=self._db)

    def filter_catalog(self, catalog):
        return self.get_queryset().filter_catalog(catalog)



from django.conf import settings
from django.contrib.auth import views as auth_views
from django.urls import include, re_path

from ..views import profile_update, remove_user, terms_of_use

urlpatterns = [
    # edit own profile
    re_path(r'^$', profile_update, name='profile_update'),
    re_path('^remove', remove_user, name='profile_remove'),
]

if settings.ACCOUNT_TERMS_OF_USE is True:
    urlpatterns += [
        re_path('^terms-of-use/', terms_of_use, name='terms_of_use')
    ]

if settings.ACCOUNT or settings.SOCIALACCOUNT:
    # include django-allauth urls
    urlpatterns += [
        re_path(r'^', include('allauth.urls'))
    ]
elif settings.SHIBBOLETH:
    urlpatterns += [
        re_path('^logout/', auth_views.LogoutView.as_view(next_page=settings.SHIBBOLETH_LOGOUT_URL), name='account_logout'),
    ]
else:
    urlpatterns += [
        re_path('^login/', auth_views.LoginView.as_view(template_name='account/login.html'), name='account_login'),
        re_path('^logout/', auth_views.LogoutView.as_view(next_page=settings.LOGIN_REDIRECT_URL), name='account_logout'),
    ]



from rdmo.core.serializers import TranslationSerializerMixin
from rest_framework import serializers

from ..models import Task


class TaskExportSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    start_attribute = serializers.CharField(source='start_attribute.uri', default=None, read_only=True)
    end_attribute = serializers.CharField(source='end_attribute.uri', default=None, read_only=True)
    conditions = serializers.SerializerMethodField()
    catalogs = serializers.SerializerMethodField()

    class Meta:
        model = Task
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'start_attribute',
            'end_attribute',
            'days_before',
            'days_after',
            'conditions',
            'catalogs'
        )
        trans_fields = (
            'title',
            'text'
        )

    def get_conditions(self, obj):
        return [condition.uri for condition in obj.conditions.all()]

    def get_catalogs(self, obj):
        return [catalog.uri for catalog in obj.catalogs.all()]



import io
import os

import pytest
from django.core.management import call_command
from django.core.management.base import CommandError

files = (
    'conditions.xml',
    'domain.xml',
    'options.xml',
    'questions.xml',
    'tasks.xml',
    'views.xml'
)


@pytest.mark.parametrize('file_name', files)
def test_import(db, settings, file_name):
    xml_file = os.path.join(settings.BASE_DIR, 'xml', file_name)
    stdout, stderr = io.StringIO(), io.StringIO()

    call_command('import', xml_file, stdout=stdout, stderr=stderr)

    assert not stdout.getvalue()
    assert not stderr.getvalue()


def test_import_error(db, settings):
    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'error.xml')
    stdout, stderr = io.StringIO(), io.StringIO()

    with pytest.raises(CommandError) as e:
        call_command('import', xml_file, stdout=stdout, stderr=stderr)

    assert str(e.value) == 'The content of the xml file does not consist of well formed data or markup.'


def test_import_error2(db, settings):
    xml_file = os.path.join(settings.BASE_DIR, 'xml', 'project.xml')
    stdout, stderr = io.StringIO(), io.StringIO()

    with pytest.raises(CommandError) as e:
        call_command('import', xml_file, stdout=stdout, stderr=stderr)

    assert str(e.value) == 'This XML does not contain RDMO content.'



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Option, OptionSet
from ..serializers.v1 import OptionSerializer
from ..validators import OptionUniqueURIValidator


def test_unique_uri_validator_create(db):
    OptionUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'optionset': OptionSet.objects.first()
    })


def test_unique_uri_validator_create_error(db):
    optionset = OptionSet.objects.first()

    with pytest.raises(ValidationError):
        OptionUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': optionset.options.first().key,
            'optionset': optionset
        })


def test_unique_uri_validator_update(db):
    option = Option.objects.first()

    OptionUniqueURIValidator(option)({
        'uri_prefix': option.uri_prefix,
        'key': option.key,
        'optionset': option.optionset
    })


def test_unique_uri_validator_update_error(db):
    option = Option.objects.first()

    with pytest.raises(ValidationError):
        OptionUniqueURIValidator(option)({
            'uri_prefix': option.uri_prefix,
            'key': option.optionset.options.last().key,
            'optionset': option.optionset
        })


def test_unique_uri_validator_serializer_create(db):
    validator = OptionUniqueURIValidator()
    validator.set_context(OptionSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'optionset': OptionSet.objects.first()
    })


def test_unique_uri_validator_serializer_create_error(db):
    optionset = OptionSet.objects.first()

    validator = OptionUniqueURIValidator()
    validator.set_context(OptionSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': optionset.options.last().key,
            'optionset': optionset
        })


def test_unique_uri_validator_serializer_update(db):
    option = Option.objects.first()

    validator = OptionUniqueURIValidator()
    validator.set_context(OptionSerializer(instance=option))

    validator({
        'uri_prefix': option.uri_prefix,
        'key': option.key,
        'optionset': option.optionset
    })


def test_unique_uri_validator_serializer_update_error(db):
    option = Option.objects.first()

    validator = OptionUniqueURIValidator()
    validator.set_context(OptionSerializer(instance=option))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': option.uri_prefix,
            'key': option.optionset.options.last().key,
            'optionset': option.optionset
        })



import logging

from rdmo.core.imports import (get_foreign_field, set_common_fields,
                               validate_instance)
from rdmo.domain.models import Attribute
from rdmo.options.models import Option

from .models import Condition
from .validators import ConditionLockedValidator, ConditionUniqueURIValidator

logger = logging.getLogger(__name__)


def import_condition(element, save=False):
    try:
        condition = Condition.objects.get(uri=element.get('uri'))
    except Condition.DoesNotExist:
        condition = Condition()

    set_common_fields(condition, element)

    condition.source = get_foreign_field(condition, element.get('source'), Attribute)
    condition.target_option = get_foreign_field(condition, element.get('target_option'), Option)

    condition.relation = element.get('relation')
    condition.target_text = element.get('target_text') or ''

    if save and validate_instance(condition, ConditionLockedValidator, ConditionUniqueURIValidator):
        if condition.id:
            logger.info('Catalog created with uri %s.', element.get('uri'))
        else:
            logger.info('Catalog %s updated.', element.get('uri'))

        condition.save()
        condition.imported = True

    return condition



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-27 15:28
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('views', '0002_view_template'),
    ]

    operations = [
        migrations.RenameField(
            model_name='view',
            old_name='description',
            new_name='comment',
        ),
        migrations.RenameField(
            model_name='view',
            old_name='title',
            new_name='key',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-05 15:22
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0032_remove_unit_and_type'),
        ('questions', '0018_data_migration'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='attribute',
            name='optionsets',
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-29 12:08

from django.conf import settings
from django.db import migrations, models


def run_data_migration(apps, schema_editor):
    for element in apps.get_model('options', 'OptionSet').objects.all():
        element.uri_prefix = element.uri_prefix or settings.DEFAULT_URI_PREFIX
        element.save()

    for element in apps.get_model('options', 'Option').objects.all():
        element.uri_prefix = element.uri_prefix or settings.DEFAULT_URI_PREFIX
        element.save()


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0022_cascade_options'),
    ]

    operations = [
        migrations.RunPython(run_data_migration),
        migrations.AlterField(
            model_name='option',
            name='uri_prefix',
            field=models.URLField(help_text='The prefix for the URI of this option.', max_length=256, verbose_name='URI Prefix'),
        ),
        migrations.AlterField(
            model_name='optionset',
            name='uri_prefix',
            field=models.URLField(help_text='The prefix for the URI of this option set.', max_length=256, verbose_name='URI Prefix'),
        ),
    ]



from django.utils.translation import gettext_lazy as _

ROLE_OWNER = 'owner'
ROLE_MANAGER = 'manager'
ROLE_AUTHOR = 'author'
ROLE_GUEST = 'guest'
ROLE_CHOICES = (
    (ROLE_OWNER, _('Owner')),
    (ROLE_MANAGER, _('Manager')),
    (ROLE_AUTHOR, _('Author')),
    (ROLE_GUEST, _('Guest')),
)



from django.urls import re_path

from ..views import ViewsExportView, ViewsView

urlpatterns = [
    re_path(r'^$', ViewsView.as_view(), name='views'),
    re_path(r'^export/(?P<format>[a-z]+)/$', ViewsExportView.as_view(), name='views_export'),
]



from django import template
from django.conf import settings
from django.template import TemplateDoesNotExist
from django.template.defaultfilters import stringfilter
from django.template.loader import get_template, render_to_string
from django.urls import resolve, reverse
from django.urls.resolvers import Resolver404
from django.utils import translation
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.translation import get_language, to_locale

from rdmo import __version__
from rdmo.core.utils import markdown2html

register = template.Library()


@register.simple_tag()
def i18n_switcher():
    string = ''
    for language, language_string in settings.LANGUAGES:
        url = reverse('i18n_switcher', args=[language])
        if language == translation.get_language():
            string += "<li><a href=\"%s\"><u>%s</u></a></li>" % (url, language_string)
        else:
            string += "<li><a href=\"%s\">%s</a></li>" % (url, language_string)
    return mark_safe(string)


@register.simple_tag()
def render_lang_template(template_name, escape_html=False):
    loc = to_locale(get_language())
    lst = [
        template_name + '_' + loc + '.html',
        template_name + '_' + settings.LANGUAGES[0][0] + '.html',
        template_name + '_en.html',
        template_name + '.html'
    ]
    for el in lst:
        try:
            template = get_template(el)
            html = template.render()
            if escape_html:
                return escape(html)
            else:
                return html
        except TemplateDoesNotExist:
            pass
    return ''


@register.simple_tag()
@stringfilter
def vendor(vendor_key):
    vendor_config = settings.VENDOR[vendor_key]

    tags = []

    if 'js' in vendor_config:
        for file in vendor_config['js']:
            if settings.VENDOR_CDN:
                tag = '<script src="%(url)s/%(path)s" integrity="%(sri)s" crossorigin="anonymous"></script>' % {
                    'url': vendor_config['url'].rstrip('/'),
                    'path': file['path'],
                    'sri': file['sri'] if 'sri' in file else ''
                }
            else:
                tag = '<script src="%(static_url)s/%(vendor_key)s/%(path)s"></script>' % {
                    'static_url': settings.STATIC_URL.rstrip('/'),
                    'vendor_key': vendor_key,
                    'path': file['path']
                }

            tags.append(tag)

    if 'css' in vendor_config:
        for file in vendor_config['css']:
            if settings.VENDOR_CDN:
                tag = '<link rel="stylesheet" href="%(url)s/%(path)s" integrity="%(sri)s" crossorigin="anonymous" />' % {
                    'url': vendor_config['url'].rstrip('/'),
                    'path': file['path'],
                    'sri': file['sri'] if 'sri' in file else ''
                }
            else:
                tag = '<link rel="stylesheet" href="%(static_url)s/%(vendor_key)s/%(path)s" />' % {
                    'static_url': settings.STATIC_URL.rstrip('/'),
                    'vendor_key': vendor_key,
                    'path': file['path']
                }

            tags.append(tag)

    return mark_safe(''.join(tags))


@register.simple_tag(takes_context=True)
def bootstrap_form(context, **kwargs):
    form_context = {}

    if 'form' in kwargs:
        form_context['form'] = kwargs['form']
    else:
        form_context['form'] = context['form']

    if 'next' in kwargs:
        form_context['next'] = kwargs['next']
    elif 'next' in context:
        form_context['next'] = context['next']

    if 'action_url_name' in kwargs:
        form_context['action'] = reverse(kwargs['action_url_name'])

    if 'submit' in kwargs:
        form_context['submit'] = kwargs['submit']

    return render_to_string('core/bootstrap_form.html', form_context, request=context.request)


@register.simple_tag(takes_context=True)
def bootstrap_delete_form(context, **kwargs):
    form_context = {}

    if 'next' in kwargs:
        form_context['next'] = kwargs['next']
    elif 'next' in context:
        form_context['next'] = context['next']

    if 'action_url_name' in kwargs:
        form_context['action'] = reverse(kwargs['action_url_name'])

    if 'submit' in kwargs:
        form_context['submit'] = kwargs['submit']

    return render_to_string('core/bootstrap_delete_form.html', form_context, request=context.request)


@register.simple_tag(takes_context=True)
def back_to_project_link(context):
    try:
        resolver_match = resolve(context.request.path_info)
        if resolver_match.url_name != 'project':
            project = context.get('project')
            if project:
                project_id = project.id
            else:
                project_id = resolver_match.kwargs.get('project_id')

            if project_id is not None:
                return render_to_string('core/back_to_project_link.html', {
                    'project_id': project_id
                }, request=context.request)
    except Resolver404:
        pass

    return ''


@register.filter(is_safe=True)
@stringfilter
def markdown(value):
    return mark_safe(markdown2html(value))


@register.simple_tag
def version():
    return __version__


@register.filter('startswith')
@stringfilter
def startswith(text, starts):
    if isinstance(text, str):
        return text.startswith(starts)
    return False



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-04 14:10
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0013_permissions'),
    ]

    operations = [
        migrations.AlterField(
            model_name='catalog',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional internal information about this catalog.', null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='catalog',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this catalog.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional internal information about this question/questionset.', null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this question/questionset.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='questionentity',
            name='subsection',
            field=models.ForeignKey(help_text='The subsection this question/questionset belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='entities', to='questions.Subsection', verbose_name='Subsection'),
        ),
        migrations.AlterField(
            model_name='section',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional internal information about this section.', null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='section',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this section.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='section',
            name='order',
            field=models.IntegerField(default=0, help_text='Position in lists.', verbose_name='Order'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional internal information about this subsection.', null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this subsection.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='subsection',
            name='order',
            field=models.IntegerField(default=0, help_text='Position in lists.', verbose_name='Order'),
        ),
    ]



from django.utils.translation import gettext_lazy as _

from rdmo.core.validators import LockedValidator, UniqueURIValidator

from .models import Condition


class ConditionUniqueURIValidator(UniqueURIValidator):

    model = Condition

    def get_uri(self, data):
        if not data.get('key'):
            self.raise_validation_error({'key': _('This field is required.')})
        else:
            uri = self.model.build_uri(data.get('uri_prefix'), data.get('key'))
            return uri


class ConditionLockedValidator(LockedValidator):

    pass



from rdmo.core.renderers import BaseXMLRenderer


class ConditionsRenderer(BaseXMLRenderer):

    def render_condition(self, xml, condition):
        xml.startElement('condition', {'dc:uri': condition['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, condition['uri_prefix'])
        self.render_text_element(xml, 'key', {}, condition['key'])
        self.render_text_element(xml, 'dc:comment', {}, condition['comment'])
        self.render_text_element(xml, 'source', {'dc:uri': condition['source']}, None)
        self.render_text_element(xml, 'relation', {}, condition['relation'])
        self.render_text_element(xml, 'target_text', {}, condition['target_text'])
        self.render_text_element(xml, 'target_option', {'dc:uri': condition['target_option']}, None)
        xml.endElement('condition')


class ConditionRenderer(ConditionsRenderer):

    def render_document(self, xml, conditions):
        xml.startElement('rdmo', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        for condition in conditions:
            self.render_condition(xml, condition)
        xml.endElement('rdmo')



# Generated by Django 2.2.18 on 2021-05-31 11:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0049_invite'),
    ]

    operations = [
        migrations.AddField(
            model_name='value',
            name='set_prefix',
            field=models.CharField(blank=True, default='', help_text='The position of this value with respect to superior sets (i.e. for nested question sets).', max_length=16, verbose_name='Set prefix'),
        ),
        migrations.AlterField(
            model_name='value',
            name='collection_index',
            field=models.IntegerField(default=0, help_text='The position of this value in a list (i.e. for a question tagged as collection).', verbose_name='Collection index'),
        ),
        migrations.AlterField(
            model_name='value',
            name='set_index',
            field=models.IntegerField(default=0, help_text='The position of this value in a set (i.e. for a question set tagged as collection).', verbose_name='Set index'),
        ),
    ]



from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.utils.translation import gettext_lazy as _

from rdmo.core.validators import (InstanceValidator, LockedValidator,
                                  UniqueURIValidator)

from .models import Catalog, Question, QuestionSet, Section


class CatalogUniqueURIValidator(UniqueURIValidator):

    model = Catalog

    def get_uri(self, data):
        if not data.get('key'):
            self.raise_validation_error({'key': _('This field is required.')})
        else:
            uri = self.model.build_uri(data.get('uri_prefix'), data.get('key'))
            return uri


class SectionUniqueURIValidator(UniqueURIValidator):

    model = Section

    def get_uri(self, data):
        if not data.get('key'):
            self.raise_validation_error({'key': _('This field is required.')})
        elif not data.get('catalog'):
            self.raise_validation_error({'catalog': _('This field may not be null.')})
        else:
            path = self.model.build_path(data.get('key'), data.get('catalog'))
            uri = self.model.build_uri(data.get('uri_prefix'), path)
            return uri


class QuestionSetUniqueURIValidator(UniqueURIValidator):

    def __call__(self, data):
        uri = self.get_uri(data)
        self.validate(QuestionSet, self.instance, uri)
        self.validate(Question, self.instance, uri)

    def get_uri(self, data):
        if not data.get('key'):
            self.raise_validation_error({'key': _('This field is required.')})
        elif not data.get('section'):
            self.raise_validation_error({'section': _('This field may not be null.')})
        else:
            path = QuestionSet.build_path(data.get('key'), data.get('section'), data.get('questionset'))
            uri = QuestionSet.build_uri(data.get('uri_prefix'), path)
            return uri


class QuestionUniqueURIValidator(UniqueURIValidator):

    def __call__(self, data):
        uri = self.get_uri(data)
        self.validate(Question, self.instance, uri)
        self.validate(QuestionSet, self.instance, uri)

    def get_uri(self, data):
        if not data.get('key'):
            self.raise_validation_error({'key': _('This field is required.')})
        elif not data.get('questionset'):
            self.raise_validation_error({'questionset': _('This field may not be null.')})
        else:
            path = Question.build_path(data.get('key'), data.get('questionset'))
            uri = Question.build_uri(data.get('uri_prefix'), path)
            return uri


class QuestionSetQuestionSetValidator(InstanceValidator):

    def __call__(self, data):
        questionset = data.get('questionset')
        if questionset:
            if self.serializer:
                # check copied attributes
                view = self.serializer.context.get('view')
                if view and view.action == 'copy':
                    # get the original from the view when cloning an attribute
                    if questionset in view.get_object().get_descendants(include_self=True):
                        self.raise_validation_error({
                            'questionset': [_('A question set may not be cloned to be a child of itself or one of its descendants.')]
                        })

            # only check updated attributes
            if self.instance:
                if questionset in self.instance.get_descendants(include_self=True):
                    self.raise_validation_error({
                        'questionset': [_('A question set may not be moved to be a child of itself or one of its descendants.')]
                    })



class CatalogLockedValidator(LockedValidator):

    pass


class SectionLockedValidator(LockedValidator):

    parent_field = 'catalog'


class QuestionSetLockedValidator(LockedValidator):

    parent_field = 'section'


class QuestionLockedValidator(LockedValidator):

    parent_field = 'questionset'



import subprocess

from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def handle(self, *args, **options):

        call_command('makemessages', '-l', 'de', ignore=['env', 'env2', 'env3', 'htmlcov'])
        try:
            subprocess.call(['poedit', 'locale/de/LC_MESSAGES/django.po'])
            call_command('compilemessages', '-l', 'de')
        except OSError:
            pass



from ..models import Catalog, Question, QuestionSet, Section


def test_catalog_str(db):
    instances = Catalog.objects.all()
    for instance in instances:
        assert str(instance)


def test_catalog_clean(db):
    instances = Catalog.objects.all()
    for instance in instances:
        instance.clean()


def test_catalog_copy(db):
    instances = Catalog.objects.all()
    for instance in instances:
        new_uri_prefix = instance.uri_prefix + '-'
        new_key = instance.key + '-'
        new_instance = instance.copy(new_uri_prefix, new_key)
        assert new_instance.uri_prefix == new_uri_prefix
        assert new_instance.key == new_key
        assert list(new_instance.sites.values('id')) == list(new_instance.sites.values('id'))
        assert list(new_instance.groups.values('id')) == list(new_instance.groups.values('id'))
        assert new_instance.sections.count() == instance.sections.count()


def test_section_str(db):
    instances = Section.objects.all()
    for instance in instances:
        assert str(instance)


def test_section_clean(db):
    instances = Section.objects.all()
    for instance in instances:
        instance.clean()


def test_section_copy(db):
    instances = Section.objects.all()
    for instance in instances:
        new_uri_prefix = instance.uri_prefix + '-'
        new_key = instance.key + '-'
        new_instance = instance.copy(new_uri_prefix, new_key)
        assert new_instance.uri_prefix == new_uri_prefix
        assert new_instance.key == new_key
        assert new_instance.questionsets.count() == instance.questionsets.count()


def test_questionset_str(db):
    instances = QuestionSet.objects.all()
    for instance in instances:
        assert str(instance)


def test_questionset_clean(db):
    instances = QuestionSet.objects.all()
    for instance in instances:
        instance.clean()


def test_questionset_copy(db):
    instances = QuestionSet.objects.all()
    for instance in instances:
        new_uri_prefix = instance.uri_prefix + '-'
        new_key = instance.key + '-'
        new_instance = instance.copy(new_uri_prefix, new_key)
        assert new_instance.uri_prefix == new_uri_prefix
        assert new_instance.key == new_key
        assert new_instance.attribute == instance.attribute
        assert list(new_instance.conditions.values('id')) == list(new_instance.conditions.values('id'))
        assert new_instance.questions.count() == instance.questions.count()


def test_question_str(db):
    instances = Question.objects.all()
    for instance in instances:
        assert str(instance)


def test_question_clean(db):
    instances = Question.objects.all()
    for instance in instances:
        instance.clean()


def test_question_copy(db):
    instances = Question.objects.all()
    for instance in instances:
        new_uri_prefix = instance.uri_prefix + '-'
        new_key = instance.key + '-'
        new_instance = instance.copy(new_uri_prefix, new_key)
        assert new_instance.uri_prefix == new_uri_prefix
        assert new_instance.key == new_key
        assert new_instance.attribute == instance.attribute
        assert list(new_instance.conditions.values('id')) == list(new_instance.conditions.values('id'))
        assert list(new_instance.optionsets.values('id')) == list(new_instance.optionsets.values('id'))



from django.urls import re_path

from ..views import ConditionsExportView, ConditionsView

urlpatterns = [
    re_path(r'^$', ConditionsView.as_view(), name='conditions'),
    re_path(r'^export/(?P<format>[a-z]+)/$', ConditionsExportView.as_view(), name='conditions_export'),
]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class AccountsConfig(AppConfig):
    name = 'rdmo.accounts'
    verbose_name = _('Accounts')

    def ready(self):
        from . import rules



# Generated by Django 2.2.13 on 2020-10-02 16:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0041_value_external_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='site',
            field=models.ForeignKey(help_text='The site this project belongs to (in a multi site setup).', null=True, on_delete=django.db.models.deletion.SET_NULL, to='sites.Site', verbose_name='Site'),
        ),
    ]



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Attribute
from ..serializers.v1 import AttributeSerializer
from ..validators import AttributeLockedValidator


def test_create(db):
    AttributeLockedValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'parent': Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    })


def test_create_no_parent(db):
    AttributeLockedValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_create_error(db):
    locked_attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    locked_attribute.locked = True
    locked_attribute.save()

    with pytest.raises(ValidationError):
        AttributeLockedValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': 'test',
            'parent': Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
        })


def test_update(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')

    AttributeLockedValidator(attribute)({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent': attribute.parent
    })


def test_update_error(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    attribute.locked = True
    attribute.save()

    with pytest.raises(ValidationError):
        AttributeLockedValidator(attribute)({
            'uri_prefix': attribute.uri_prefix,
            'key': attribute.key,
            'parent': attribute.parent,
            'locked': True
        })


def test_update_lock(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')

    AttributeLockedValidator(attribute)({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent':  attribute.parent,
        'locked': True
    })


def test_update_unlock(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    attribute.locked = True
    attribute.save()

    AttributeLockedValidator(attribute)({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent':  attribute.parent,
        'locked': False
    })


def test_update_error_parent(db):
    locked_attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    locked_attribute.locked = True
    locked_attribute.save()

    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    with pytest.raises(ValidationError):
        AttributeLockedValidator(attribute)({
            'uri_prefix': attribute.uri_prefix,
            'key': attribute.key,
            'parent': attribute.parent
        })


def test_serializer_create(db):
    validator = AttributeLockedValidator()
    validator.set_context(AttributeSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'parent': Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    })


def test_serializer_create_no_parent(db):
    validator = AttributeLockedValidator()
    validator.set_context(AttributeSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_serializer_create_error(db):
    locked_attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    locked_attribute.locked = True
    locked_attribute.save()

    validator = AttributeLockedValidator()
    validator.set_context(AttributeSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': 'test',
            'parent': locked_attribute
        })


def test_serializer_update(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')

    validator = AttributeLockedValidator()
    validator.set_context(AttributeSerializer(instance=attribute))

    validator({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent':  attribute.parent
    })


def test_serializer_update_error(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    attribute.locked = True
    attribute.save()

    validator = AttributeLockedValidator()
    validator.set_context(AttributeSerializer(instance=attribute))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': attribute.uri_prefix,
            'key': attribute.key,
            'parent':  attribute.parent,
            'locked': True
        })


def test_serializer_update_lock(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')

    validator = AttributeLockedValidator()
    validator.set_context(AttributeSerializer(instance=attribute))

    validator({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent':  attribute.parent,
        'locked': True
    })


def test_serializer_update_unlock(db):
    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')
    attribute.locked = True
    attribute.save()

    validator = AttributeLockedValidator()
    validator.set_context(AttributeSerializer(instance=attribute))

    validator({
        'uri_prefix': attribute.uri_prefix,
        'key': attribute.key,
        'parent':  attribute.parent,
        'locked': False
    })


def test_serializer_update_error_parent(db):
    locked_attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single')
    locked_attribute.locked = True
    locked_attribute.save()

    attribute = Attribute.objects.get(uri='http://example.com/terms/domain/individual/single/text')

    validator = AttributeLockedValidator()
    validator.set_context(AttributeSerializer(instance=attribute))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': attribute.uri_prefix,
            'key': attribute.key,
            'parent':  attribute.parent
        })



# Generated by Django 2.2.9 on 2020-02-14 10:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0050_data_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='catalog',
            name='sites',
            field=models.ManyToManyField(blank=True, help_text='The sites this catalog belongs to (in a multi site setup).', to='sites.Site', verbose_name='Sites'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-06-22 15:10
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0001_initial_after_reset'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='catalog',
            field=models.ForeignKey(help_text='The catalog which will be used for this project.', on_delete=django.db.models.deletion.CASCADE, related_name='+', to='questions.Catalog', verbose_name='catalog'),
        ),
        migrations.AlterField(
            model_name='project',
            name='description',
            field=models.TextField(blank=True, help_text='You can use markdown syntax in the description.', verbose_name='description'),
        ),
        migrations.AlterField(
            model_name='project',
            name='title',
            field=models.CharField(max_length=256, verbose_name='title'),
        ),
    ]



from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import EmailMessage


def send_mail(subject, message, from_email=None, to=[], cc=[], bcc=[], reply_to=None, attachments=[]):
    site = Site.objects.get_current()
    subject = '[{}] '.format(site.name) + subject

    if from_email is None:
        from_email = settings.DEFAULT_FROM_EMAIL

    mail = EmailMessage(subject, message, from_email, to=to, cc=cc, bcc=bcc, reply_to=reply_to, attachments=attachments)
    mail.send()



import hmac
import json
import logging
from urllib.parse import quote, urlencode

import requests
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse
from django.utils.crypto import get_random_string
from django.utils.translation import gettext_lazy as _
from rdmo.core.plugins import Plugin

logger = logging.getLogger(__name__)


class Provider(Plugin):

    def send_issue(self, request, issue, integration, subject, message, attachments):
        raise NotImplementedError

    def webhook(self, request, options, payload):
        raise NotImplementedError


class OauthProvider(Provider):

    def send_issue(self, request, issue, integration, subject, message, attachments):
        url = self.get_post_url(request, issue, integration, subject, message, attachments)
        data = self.get_post_data(request, issue, integration, subject, message, attachments)

        if url is None or data is None:
            return render(request, 'core/error.html', {
                'title': _('Integration error'),
                'errors': [_('The Integration is not configured correctly.') % message]
            }, status=200)

        return self.post(request, url, data, issue.id, integration.id)

    def post(self, request, url, data, issue_id, integration_id):
        # get access token from the session
        access_token = self.get_from_session(request, 'access_token')
        if access_token:
            logger.debug('post: %s %s', url, data)

            response = requests.post(url, json=data, headers=self.get_authorization_headers(access_token))

            if response.status_code == 401:
                logger.warn('post forbidden: %s (%s)', response.content, response.status_code)
            else:
                try:
                    response.raise_for_status()
                    remote_url = self.get_issue_url(response)
                    self.update_issue(issue_id, integration_id, remote_url)
                    return HttpResponseRedirect(remote_url)
                except requests.HTTPError:
                    logger.warn('post error: %s (%s)', response.content, response.status_code)

                    message = response.json().get('error')
                    return render(request, 'core/error.html', {
                        'title': _('Send error'),
                        'errors': [_('Something went wrong: %s.') % message]
                    }, status=200)

        # if the above did not work authorize first
        self.store_in_session(request, 'post', (url, data, issue_id, integration_id))
        return self.authorize(request)

    def authorize(self, request):
        # get random state and store in session
        state = get_random_string(length=32)
        self.store_in_session(request, 'state', state)

        url = self.authorize_url + '?' + urlencode(self.get_authorize_params(request, state))
        return HttpResponseRedirect(url)

    def callback(self, request):
        assert request.GET.get('state') == self.pop_from_session(request, 'state')

        url = self.token_url + '?' + urlencode(self.get_callback_params(request))

        response = requests.post(url, headers={
            'Accept': 'application/json'
        })

        try:
            response.raise_for_status()
        except requests.HTTPError as e:
            logger.error('callback error: %s (%s)', response.content, response.status_code)
            raise e

        response_data = response.json()

        # store access token in session
        self.store_in_session(request, 'access_token', response_data.get('access_token'))

        # get post data from session
        try:
            url, data, issue_id, integration_id = self.pop_from_session(request, 'post')
            return self.post(request, url, data, issue_id, integration_id)
        except ValueError:
            return render(request, 'core/error.html', {
                'title': _('Authorization successful'),
                'errors': [_('But no redirect could be found.')]
            }, status=200)

    def update_issue(self, issue_id, integration_id, remote_url):
        from rdmo.projects.models import Issue, Integration, IssueResource
        try:
            issue = Issue.objects.get(pk=issue_id)
            issue.status = Issue.ISSUE_STATUS_IN_PROGRESS
            issue.save()

            integration = Integration.objects.get(pk=integration_id)

            issue_resource = IssueResource(issue=issue, integration=integration, url=remote_url)
            issue_resource.save()
        except ObjectDoesNotExist:
            pass

    def get_session_key(self, key):
        class_name = self.__class__.__name__.lower()
        return '{}_{}'.format(class_name, key)

    def store_in_session(self, request, key, data):
        session_key = self.get_session_key(key)
        request.session[session_key] = data

    def get_from_session(self, request, key):
        session_key = self.get_session_key(key)
        return request.session.get(session_key, None)

    def pop_from_session(self, request, key):
        session_key = self.get_session_key(key)
        return request.session.pop(session_key, None)

    def get_post_url(self, request, issue, integration, subject, message, attachments):
        raise NotImplementedError

    def get_post_data(self, request, issue, integration, subject, message, attachments):
        raise NotImplementedError

    def get_authorization_headers(self, access_token):
        raise NotImplementedError

    def get_authorize_params(self, request, state):
        raise NotImplementedError

    def get_callback_params(self, request):
        raise NotImplementedError

    def get_issue_url(self, response):
        raise NotImplementedError


class GitHubProvider(OauthProvider):
    add_label = _('Add GitHub integration')
    send_label = _('Send to GitHub')
    description = _('This integration allow the creation of issues in arbitrary GitHub repositories. '
                    'The upload of attachments is not supported by GitHub.')

    authorize_url = 'https://github.com/login/oauth/authorize'
    token_url = 'https://github.com/login/oauth/access_token'

    @property
    def client_id(self):
        return settings.GITHUB_PROVIDER['client_id']

    @property
    def client_secret(self):
        return settings.GITHUB_PROVIDER['client_secret']

    @property
    def redirect_path(self):
        return reverse('oauth_callback', args=['github'])

    def get_repo(self, integration):
        try:
            return integration.options.get(key='repo').value
        except ObjectDoesNotExist:
            return None

    def get_secret(self, integration):
        try:
            return integration.options.get(key='secret').value
        except ObjectDoesNotExist:
            return None

    def get_post_url(self, request, issue, integration, subject, message, attachments):
        repo = self.get_repo(integration)
        if repo:
            return 'https://api.github.com/repos/{}/issues'.format(repo)

    def get_post_data(self, request, issue, integration, subject, message, attachments):
        return {
            'title': subject,
            'body': message
        }

    def get_authorization_headers(self, access_token):
        return {
            'Authorization': 'token {}'.format(access_token),
            'Accept': 'application/vnd.github.v3+json'
        }

    def get_authorize_params(self, request, state):
        return {
            'authorize_url': self.authorize_url,
            'client_id': self.client_id,
            'redirect_uri': request.build_absolute_uri(self.redirect_path),
            'scope': 'repo',
            'state': state,
        }

    def get_callback_params(self, request):
        return {
            'token_url': self.token_url,
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'code': request.GET.get('code')
        }

    def get_issue_url(self, response):
        return response.json().get('html_url')

    def webhook(self, request, integration):
        secret = self.get_secret(integration)
        header_signature = request.headers.get('X-Hub-Signature')

        if (secret is not None) and (header_signature is not None):
            body_signature = 'sha1=' + hmac.new(secret.encode(), request.body, 'sha1').hexdigest()

            if hmac.compare_digest(header_signature, body_signature):
                try:
                    payload = json.loads(request.body.decode())
                    action = payload.get('action')
                    issue_url = payload.get('issue', {}).get('html_url')

                    if action and issue_url:
                        try:
                            issue_resource = integration.resources.get(url=issue_url)
                            if action == 'closed':
                                issue_resource.issue.status = issue_resource.issue.ISSUE_STATUS_CLOSED
                            else:
                                issue_resource.issue.status = issue_resource.issue.ISSUE_STATUS_IN_PROGRESS

                            issue_resource.issue.save()
                        except ObjectDoesNotExist:
                            pass

                    return HttpResponse(status=200)

                except json.decoder.JSONDecodeError as e:
                    return HttpResponse(e, status=400)

        raise Http404

    @property
    def fields(self):
        return [
            {
                'key': 'repo',
                'placeholder': 'user_name/repo_name',
                'help': _('The GitHub repository to send issues to.')
            },
            {
                'key': 'secret',
                'placeholder': 'Secret (random) string',
                'help': _('The secret for a GitHub webhook to close a task.'),
                'required': False,
                'secret': True
            }
        ]


class GitLabProvider(OauthProvider):
    add_label = _('Add GitLab integration')
    send_label = _('Send to GitLab')

    @property
    def gitlab_url(self):
        return settings.GITLAB_PROVIDER['gitlab_url'].strip('/')

    @property
    def description(self):
        return _('This integration allow the creation of issues in arbitrary repositories on {}. '
                 'The upload of attachments is not supported by GitLab.'.format(self.gitlab_url))

    @property
    def authorize_url(self):
        return '{}/oauth/authorize'.format(self.gitlab_url)

    @property
    def token_url(self):
        return '{}/oauth/token'.format(self.gitlab_url)

    @property
    def client_id(self):
        return settings.GITLAB_PROVIDER['client_id']

    @property
    def client_secret(self):
        return settings.GITLAB_PROVIDER['client_secret']

    @property
    def redirect_path(self):
        return reverse('oauth_callback', args=['gitlab'])

    def get_repo(self, integration):
        try:
            return integration.options.get(key='repo').value
        except ObjectDoesNotExist:
            return None

    def get_secret(self, integration):
        try:
            return integration.options.get(key='secret').value
        except ObjectDoesNotExist:
            return None

    def get_post_url(self, request, issue, integration, subject, message, attachments):
        repo = self.get_repo(integration)
        if repo:
            return '{}/api/v4/projects/{}/issues'.format(self.gitlab_url, quote(repo, safe=''))

    def get_post_data(self, request, issue, integration, subject, message, attachments):
        return {
            'title': subject,
            'description': message
        }

    def get_authorization_headers(self, access_token):
        return {
            'Authorization': 'Bearer {}'.format(access_token)
        }

    def get_authorize_params(self, request, state):
        return {
            'authorize_url': self.authorize_url,
            'client_id': self.client_id,
            'redirect_uri': request.build_absolute_uri(self.redirect_path),
            'response_type': 'code',
            'scope': 'api',
            'state': state,
        }

    def get_callback_params(self, request):
        return {
            'token_url': self.token_url,
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'code': request.GET.get('code'),
            'grant_type': 'authorization_code',
            'redirect_uri': request.build_absolute_uri(self.redirect_path)
        }

    def get_issue_url(self, response):
        return response.json().get('web_url')

    def webhook(self, request, integration):
        secret = self.get_secret(integration)
        header_token = request.headers.get('X-Gitlab-Token')

        if (secret is not None) and (header_token is not None) and (header_token == secret):
            try:
                payload = json.loads(request.body.decode())
                state = payload.get('object_attributes', {}).get('state')
                issue_url = payload.get('object_attributes', {}).get('url')

                if state and issue_url:
                    try:
                        issue_resource = integration.resources.get(url=issue_url)
                        if state == 'closed':
                            issue_resource.issue.status = issue_resource.issue.ISSUE_STATUS_CLOSED
                        else:
                            issue_resource.issue.status = issue_resource.issue.ISSUE_STATUS_IN_PROGRESS

                        issue_resource.issue.save()
                    except ObjectDoesNotExist:
                        pass

                return HttpResponse(status=200)

            except json.decoder.JSONDecodeError as e:
                return HttpResponse(e, status=400)

        raise Http404

    @property
    def fields(self):
        return [
            {
                'key': 'repo',
                'placeholder': 'user_name/repo_name',
                'help': _('The GitLab repository to send issues to.')
            },
            {
                'key': 'secret',
                'placeholder': 'Secret (random) string',
                'help': _('The secret for a GitLab webhook to close a task.'),
                'required': False,
                'secret': True
            }
        ]



# Generated by Django 2.2.13 on 2020-08-24 11:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0020_require_uri_prefix'),
    ]

    operations = [
        migrations.AlterField(
            model_name='condition',
            name='source',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='The attribute of the value for this condition.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='conditions', to='domain.Attribute', verbose_name='Source'),
        ),
        migrations.AlterField(
            model_name='condition',
            name='target_option',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='If using a value pointing to an option, the option this condition is checking against.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='conditions', to='options.Option', verbose_name='Target (Option)'),
        ),
    ]



from pathlib import Path

import pytest
from django.conf import settings
from django.urls import reverse

from rdmo.core.constants import VALUE_TYPE_FILE

from ..models import Project, Snapshot, Value

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

view_snapshot_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'author': [1, 3, 5],
    'guest': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

add_snapshot_permission_map = change_snapshot_permission_map = rollback_snapshot_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5]
}

projects = [1, 2, 3, 4, 5]
snapshots = [1, 3, 7, 4, 5, 6]

snapshots_project = 1


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_snapshot_create_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('snapshot_create', args=[project_id])
    response = client.get(url)

    if project_id in add_snapshot_permission_map.get(username, []):
        assert response.status_code == 200
    elif password:
        assert response.status_code == 403
    else:
        assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_snapshot_create_post(db, client, files, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    snapshot_count = project.snapshots.count()
    values_count = project.values.count()
    current_values_count = project.values.filter(snapshot=None).count()

    url = reverse('snapshot_create', args=[project.id])
    data = {
        'title': 'A new snapshot',
        'description': 'Some description'
    }
    response = client.post(url, data)

    # check if all the files are where are supposed to be
    for file_value in Value.objects.filter(value_type=VALUE_TYPE_FILE):
        assert Path(settings.MEDIA_ROOT).joinpath(file_value.file.name).exists()

    if project.id in add_snapshot_permission_map.get(username, []):
        assert response.status_code == 302
        assert project.snapshots.count() == snapshot_count + 1
        assert project.values.count() == values_count + current_values_count
        for file_value in project.values.filter(value_type=VALUE_TYPE_FILE):
            assert Path(settings.MEDIA_ROOT).joinpath(file_value.file.name).exists()
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert project.snapshots.count() == snapshot_count
        assert project.values.count() == values_count


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('snapshot_id', snapshots)
def test_snapshot_update_get(db, client, username, password, project_id, snapshot_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    url = reverse('snapshot_update', args=[project_id, snapshot_id])
    response = client.get(url)

    # check if all the files are where are supposed to be
    for file_value in Value.objects.filter(value_type=VALUE_TYPE_FILE):
        assert Path(settings.MEDIA_ROOT).joinpath(file_value.file.name).exists()

    if snapshot_id in project.snapshots.values_list('id', flat=True):
        if project_id in change_snapshot_permission_map.get(username, []):
            assert response.status_code == 200
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('snapshot_id', snapshots)
def test_snapshot_update_post(db, client, username, password, project_id, snapshot_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)
    snapshot = Snapshot.objects.get(pk=snapshot_id)

    snapshot_count = project.snapshots.count()
    values_count = project.values.count()
    values_files = [value.file.name for value in project.values.filter(value_type=VALUE_TYPE_FILE)]

    url = reverse('snapshot_update', args=[project_id, snapshot_id])
    data = {
        'title': snapshot.title,
        'description': snapshot.description
    }
    response = client.post(url, data)

    # check if all the files are where are supposed to be
    for file_value in Value.objects.filter(value_type=VALUE_TYPE_FILE):
        assert Path(settings.MEDIA_ROOT).joinpath(file_value.file.name).exists()

    if snapshot_id in Project.objects.get(pk=project_id).snapshots.values_list('id', flat=True):
        if project_id in change_snapshot_permission_map.get(username, []):
            assert response.status_code == 302
            for file_value in project.values.filter(value_type=VALUE_TYPE_FILE):
                assert Path(settings.MEDIA_ROOT).joinpath(file_value.file.name).exists()
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404

    assert Project.objects.get(pk=project_id).snapshots.count() == snapshot_count
    assert Project.objects.get(pk=project_id).values.count() == values_count
    for file_value in values_files:
        assert Path(settings.MEDIA_ROOT).joinpath(file_value).exists()


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('snapshot_id', snapshots)
def test_snapshot_rollback_get(db, client, username, password, project_id, snapshot_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)
    project_snapshots = list(project.snapshots.values_list('id', flat=True))

    url = reverse('snapshot_rollback', args=[project_id, snapshot_id])
    response = client.get(url)

    if snapshot_id in project_snapshots:
        if project_id in rollback_snapshot_permission_map.get(username, []):
            assert response.status_code == 200
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('snapshot_id', snapshots)
def test_snapshot_rollback_post(db, client, files, username, password, project_id, snapshot_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)
    project_snapshots = list(project.snapshots.values_list('id', flat=True))

    url = reverse('snapshot_rollback', args=[project_id, snapshot_id])
    response = client.post(url)

    # check if all the files are where are supposed to be
    for file_value in Value.objects.filter(value_type=VALUE_TYPE_FILE):
        assert Path(settings.MEDIA_ROOT).joinpath(file_value.file.name).exists()

    if snapshot_id in project_snapshots:
        if project_id in rollback_snapshot_permission_map.get(username, []):
            assert response.status_code == 302
        elif password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302
    else:
        assert response.status_code == 404



import logging

from django.conf import settings
from django.contrib.auth import authenticate, get_user_model
from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType

from .models import Role
from .settings import GROUPS

log = logging.getLogger(__name__)


def get_full_name(user):
    if user.first_name and user.last_name:
        return '%s %s' % (user.first_name, user.last_name)
    else:
        return user.username


def is_site_manager(user):
    if user.is_authenticated:
        if user.is_superuser:
            return True
        else:
            try:
                return user.role.manager.filter(pk=settings.SITE_ID).exists()
            except Role.DoesNotExist:
                return False
    else:
        return False


def set_group_permissions():
    for name, permissions in GROUPS:
        group = Group.objects.get(name=name)
        for app_label, model, codename in permissions:
            content_type = ContentType.objects.get(app_label=app_label, model=model)
            permission = Permission.objects.get(content_type=content_type, codename=codename)
            group.permissions.add(permission)


def delete_user(user, email, password):
    username = user.username

    try:
        database_user = get_user_model().objects.get(email=email)
    except get_user_model().DoesNotExist:
        log.debug('User with email "%s" requested for deletion does not exist', email)
        return False

    if user == database_user and authenticate(username=username, password=password):
        try:
            user.delete()
            log.debug('User "%s" deleted', username)
            return True

        except Exception as e:
            log.debug('An exception (%s) occured during user "%s" deletion: ', (str(e), username))
            return False
    else:
        log.debug('Deletion of user "%s" failed because of an invalid password', username)
        return False



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-09-28 11:23
from __future__ import unicode_literals

from django.db import migrations


def set_membership(apps, schema_editor):
    Project = apps.get_model('projects', 'Project')
    Membership = apps.get_model('projects', 'Membership')
    for project in Project.objects.all():
        for user in project.owner.all():
            membership = Membership(project=project, user=user, role='owner')
            membership.save()


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0012_membership'),
    ]

    operations = [
        migrations.RunPython(set_membership),
    ]



from django.utils.timezone import now, get_current_timezone

from rdmo.core.renderers import BaseXMLRenderer
from rdmo.core.utils import get_languages


class ViewsRenderer(BaseXMLRenderer):

    def render_view(self, xml, view):
        xml.startElement('view', {'dc:uri': view['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, view['uri_prefix'])
        self.render_text_element(xml, 'key', {}, view['key'])
        self.render_text_element(xml, 'dc:comment', {}, view['comment'])

        for lang_code, lang_string, lang_field in get_languages():
            self.render_text_element(xml, 'title', {'lang': lang_code}, view['title_%s' % lang_code])
            self.render_text_element(xml, 'help', {'lang': lang_code}, view['help_%s' % lang_code])

        xml.startElement('catalogs', {})
        if 'catalogs' in view and view['catalogs']:
            for catalog in view['catalogs']:
                self.render_text_element(xml, 'catalog', {'dc:uri': catalog}, None)
        xml.endElement('catalogs')

        self.render_text_element(xml, 'template', {}, view['template'])
        xml.endElement('view')


class ViewRenderer(ViewsRenderer):

    def render_document(self, xml, views):
        xml.startElement('rdmo', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        for view in views:
            self.render_view(xml, view)
        xml.endElement('rdmo')



from django.contrib import admin

from .models import AdditionalField, AdditionalFieldValue, ConsentFieldValue, Role


class AdditionalFieldAdmin(admin.ModelAdmin):
    pass


class AdditionalFieldValueAdmin(admin.ModelAdmin):
    readonly_fields = ('user', )


class ConsentFieldValueAdmin(admin.ModelAdmin):
    readonly_fields = ('user', 'consent')

    def has_add_permission(self, request, obj=None):
        return False


class RoleAdmin(admin.ModelAdmin):
    pass


admin.site.register(AdditionalField, AdditionalFieldAdmin)
admin.site.register(AdditionalFieldValue, AdditionalFieldValueAdmin)
admin.site.register(ConsentFieldValue, ConsentFieldValueAdmin)
admin.site.register(Role, RoleAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


def set_questionentity_is_collection(apps, schema_editor):
    Value = apps.get_model('projects', 'Value')

    for value in Value.objects.all():
        if value.value_type == 'options':
            value.value_type = 'option'

        value.save()


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0019_option'),
        ('questions', '0023_option'),
    ]

    operations = [
        migrations.RunPython(set_questionentity_is_collection),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-21 13:14
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0030_data_migration'),
    ]

    operations = [
        migrations.RenameField(
            model_name='question',
            old_name='attribute_entity',
            new_name='attribute',
        ),
        migrations.RenameField(
            model_name='questionset',
            old_name='attribute_entity',
            new_name='attribute',
        ),
    ]



import logging

from django.contrib.sites.models import Site
from rdmo.conditions.models import Condition
from rdmo.core.imports import (get_foreign_field, get_m2m_instances,
                               set_common_fields, set_lang_field,
                               validate_instance)
from rdmo.domain.models import Attribute
from rdmo.questions.models import Catalog

from .models import Task

logger = logging.getLogger(__name__)


def import_task(element, save=False):
    try:
        task = Task.objects.get(uri=element.get('uri'))
    except Task.DoesNotExist:
        task = Task()

    set_common_fields(task, element)

    set_lang_field(task, 'title', element)
    set_lang_field(task, 'text', element)

    task.start_attribute = get_foreign_field(task, element.get('start_attribute'), Attribute)
    task.end_attribute = get_foreign_field(task, element.get('end_attribute'), Attribute)

    task.days_before = element.get('days_before')
    task.days_after = element.get('days_after')

    conditions = get_m2m_instances(task, element.get('conditions'), Condition)
    catalogs = get_m2m_instances(task, element.get('catalogs'), Catalog)

    if save and validate_instance(task):
        if task.id:
            logger.info('Task created with uri %s.', element.get('uri'))
        else:
            logger.info('Task %s updated.', element.get('uri'))

        task.save()
        task.sites.add(Site.objects.get_current())
        task.catalogs.set(catalogs)
        task.conditions.set(conditions)
        task.imported = True

    return task



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-30 15:28
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0010_add_db_contraint'),
    ]

    operations = [
        migrations.AlterField(
            model_name='project',
            name='catalog',
            field=models.ForeignKey(help_text='The catalog which will be used for this project.', on_delete=django.db.models.deletion.CASCADE, related_name='+', to='questions.Catalog', verbose_name='Catalog'),
        ),
        migrations.AlterField(
            model_name='project',
            name='description',
            field=models.TextField(blank=True, help_text='A description for this project (optional).', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='project',
            name='owner',
            field=models.ManyToManyField(help_text='The list of owners for this project.', to=settings.AUTH_USER_MODEL, verbose_name='Owner'),
        ),
        migrations.AlterField(
            model_name='project',
            name='title',
            field=models.CharField(help_text='The title for this project.', max_length=256, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='snapshot',
            name='description',
            field=models.TextField(blank=True, help_text='A description for this snapshot (optional).', verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='snapshot',
            name='project',
            field=models.ForeignKey(help_text='The project this snapshot belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='snapshots', to='projects.Project', verbose_name='Project'),
        ),
        migrations.AlterField(
            model_name='snapshot',
            name='title',
            field=models.CharField(help_text='The title for this snapshot.', max_length=256, verbose_name='Title'),
        ),
        migrations.AlterField(
            model_name='value',
            name='attribute',
            field=models.ForeignKey(blank=True, help_text='The attribute this value belongs to.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='values', to='domain.Attribute', verbose_name='Attribute'),
        ),
        migrations.AlterField(
            model_name='value',
            name='collection_index',
            field=models.IntegerField(default=0, help_text='The position of this value in an attribute collection.', verbose_name='Collection index'),
        ),
        migrations.AlterField(
            model_name='value',
            name='option',
            field=models.ForeignKey(blank=True, help_text='The option stored for this value.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='options.Option', verbose_name='Option'),
        ),
        migrations.AlterField(
            model_name='value',
            name='project',
            field=models.ForeignKey(help_text='The project this value belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='values', to='projects.Project', verbose_name='Project'),
        ),
        migrations.AlterField(
            model_name='value',
            name='set_index',
            field=models.IntegerField(default=0, help_text='The position of this value in an entity collection (i.e. in the question set)', verbose_name='Set index'),
        ),
        migrations.AlterField(
            model_name='value',
            name='snapshot',
            field=models.ForeignKey(blank=True, help_text='The snapshot this value belongs to.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='values', to='projects.Snapshot', verbose_name='Snapshot'),
        ),
        migrations.AlterField(
            model_name='value',
            name='text',
            field=models.TextField(blank=True, help_text='The string stored for this value.', null=True, verbose_name='Text'),
        ),
    ]



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import Question, QuestionSet
from ..serializers.v1 import QuestionSerializer
from ..validators import QuestionUniqueURIValidator


def test_unique_uri_validator_create(db):
    QuestionUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'questionset': QuestionSet.objects.first()
    })


def test_unique_uri_validator_create_error(db):
    questionset = QuestionSet.objects.first()

    with pytest.raises(ValidationError):
        QuestionUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': questionset.questions.first().key,
            'questionset': questionset
        })


def test_unique_uri_validator_create_error2(db):
    # get a questionset which contains a questionset
    questionset = QuestionSet.objects.exclude(questionset=None).first().questionset

    with pytest.raises(ValidationError):
        QuestionUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': questionset.questionsets.first().key,
            'questionset': questionset
        })


def test_unique_uri_validator_update(db):
    question = Question.objects.first()

    QuestionUniqueURIValidator(question)({
        'uri_prefix': question.uri_prefix,
        'key': question.key,
        'questionset': question.questionset
    })


def test_unique_uri_validator_update_error(db):
    question = Question.objects.get(pk=18)

    with pytest.raises(ValidationError):
        QuestionUniqueURIValidator(question)({
            'uri_prefix': question.uri_prefix,
            'key': question.questionset.questions.last().key,
            'questionset': question.questionset
        })


def test_unique_uri_validator_serializer_create(db):
    validator = QuestionUniqueURIValidator()
    validator.set_context(QuestionSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test',
        'questionset': QuestionSet.objects.first()
    })


def test_unique_uri_validator_serializer_create_error(db):
    questionset = QuestionSet.objects.first()

    validator = QuestionUniqueURIValidator()
    validator.set_context(QuestionSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': questionset.questions.last().key,
            'questionset': questionset
        })


def test_unique_uri_validator_serializer_create_error2(db):
    # get a questionset which contains a questionset
    questionset = QuestionSet.objects.exclude(questionset=None).first().questionset

    validator = QuestionUniqueURIValidator()
    validator.set_context(QuestionSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': questionset.questionsets.first().key,
            'questionset': questionset
        })


def test_unique_uri_validator_serializer_update(db):
    question = Question.objects.first()

    validator = QuestionUniqueURIValidator()
    validator.set_context(QuestionSerializer(instance=question))

    validator({
        'uri_prefix': question.uri_prefix,
        'key': question.key,
        'questionset': question.questionset
    })


def test_unique_uri_validator_serializer_update_error(db):
    question = Question.objects.get(pk=18)

    validator = QuestionUniqueURIValidator()
    validator.set_context(QuestionSerializer(instance=question))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': question.uri_prefix,
            'key': question.questionset.questions.last().key,
            'questionset': question.questionset
        })



# Generated by Django 2.2.18 on 2021-05-27 11:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0062_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='questionset',
            name='questionset',
            field=models.ForeignKey(blank=True, default=None, help_text='The question set this question set belongs to.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='questionsets', to='questions.QuestionSet', verbose_name='Question set'),
        ),
    ]



# Generated by Django 2.2.13 on 2020-09-24 13:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0038_integration_integrationoption'),
    ]

    operations = [
        migrations.AddField(
            model_name='integrationoption',
            name='secret',
            field=models.BooleanField(default=False, help_text='Designates whether this integration option is hidden from regular users.', verbose_name='Secret'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2019-03-13 11:54
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('views', '0014_data_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='view',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional internal information about this view.', verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='view',
            name='help_lang1',
            field=models.TextField(blank=True, help_text='The help text for this view in the primary language.', verbose_name='Help (primary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='help_lang2',
            field=models.TextField(blank=True, help_text='The help text for this view in the secondary language.', verbose_name='Help (secondary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='help_lang3',
            field=models.TextField(blank=True, help_text='The help text for this view in the tertiary language.', verbose_name='Help (tertiary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='help_lang4',
            field=models.TextField(blank=True, help_text='The help text for this view in the quaternary language.', verbose_name='Help (quaternary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='help_lang5',
            field=models.TextField(blank=True, help_text='The help text for this view in the quinary language.', verbose_name='Help (quinary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this view.', max_length=128, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='view',
            name='template',
            field=models.TextField(blank=True, help_text='The template for this view, written in Django template language.', verbose_name='Template'),
        ),
        migrations.AlterField(
            model_name='view',
            name='title_lang1',
            field=models.CharField(blank=True, help_text='The title for this view in the primary language.', max_length=256, verbose_name='Title (primary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='title_lang2',
            field=models.CharField(blank=True, help_text='The title for this view in the secondary language.', max_length=256, verbose_name='Title (secondary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='title_lang3',
            field=models.CharField(blank=True, help_text='The title for this view in the tertiary language.', max_length=256, verbose_name='Title (tertiary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='title_lang4',
            field=models.CharField(blank=True, help_text='The title for this view in the quaternary language.', max_length=256, verbose_name='Title (quaternary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='title_lang5',
            field=models.CharField(blank=True, help_text='The title for this view in the quinary language.', max_length=256, verbose_name='Title (quinary)'),
        ),
        migrations.AlterField(
            model_name='view',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this view (auto-generated).', max_length=640, verbose_name='URI'),
        ),
        migrations.AlterField(
            model_name='view',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this view.', max_length=256, verbose_name='URI Prefix'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-09-28 11:23
from __future__ import unicode_literals

from django.db import migrations


def set_question_optionset(apps, schema_editor):
    Question = apps.get_model('questions', 'Question')

    for question in Question.objects.all():
        try:
            for optionset in question.attribute_entity.attribute.optionsets.all():
                question.optionsets.add(optionset)

        except AttributeError:
            pass


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0017_question_optionsets'),
    ]

    operations = [
        migrations.RunPython(set_question_optionset),
    ]



# Generated by Django 2.2.16 on 2021-01-20 16:47

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0054_meta'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('projects', '0046_project_mptt'),
    ]

    operations = [
        migrations.CreateModel(
            name='Continuation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(editable=False, verbose_name='created')),
                ('updated', models.DateTimeField(editable=False, verbose_name='updated')),
                ('project', models.ForeignKey(help_text='The project for this continuation.', on_delete=django.db.models.deletion.CASCADE, related_name='+', to='projects.Project', verbose_name='Project')),
                ('questionset', models.ForeignKey(help_text='The question set for this continuation.', on_delete=django.db.models.deletion.CASCADE, related_name='+', to='questions.QuestionSet', verbose_name='Question set')),
                ('user', models.ForeignKey(help_text='The user for this continuation.', on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Continuation',
                'verbose_name_plural': 'Continuations',
                'ordering': ('user', 'project'),
            },
        ),
    ]



# Generated by Django 2.2.18 on 2021-03-22 15:47

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Overlay',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url_name', models.SlugField(help_text='The url_name for this overlay.', max_length=128, verbose_name='Url name')),
                ('current', models.SlugField(blank=True, help_text='The current state for this overlay.', max_length=128, verbose_name='Current')),
                ('site', models.ForeignKey(help_text='The site for this overlay.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='sites.Site', verbose_name='Site')),
                ('user', models.ForeignKey(help_text='The user for this overlay.', on_delete=django.db.models.deletion.CASCADE, related_name='overlays', to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Overlay',
                'verbose_name_plural': 'Overlays',
                'ordering': ('user', 'url_name'),
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0004_permission_added'),
    ]

    operations = [
        migrations.AlterField(
            model_name='detailkey',
            name='type',
            field=models.CharField(max_length=11, choices=[('text', 'Text'), ('textarea', 'Textarea'), ('checkbox', 'Checkbox'), ('radio', 'Radio button'), ('select', 'Select'), ('multiselect', 'Multiselect')]),
        ),
    ]



import logging

from django import forms
from django.conf import settings
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from .models import AdditionalField, AdditionalFieldValue, ConsentFieldValue

log = logging.getLogger(__name__)


class ProfileForm(forms.ModelForm):

    use_required_attribute = False

    class Meta:
        model = get_user_model()
        if settings.ACCOUNT:
            fields = ('first_name', 'last_name')
        else:
            fields = ('first_name', 'last_name', 'email')

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)

        self.fields['first_name'].widget = forms.TextInput(attrs={'placeholder': _('First name')})
        self.fields['last_name'].widget = forms.TextInput(attrs={'placeholder': _('Last name')})

        self.additional_fields = AdditionalField.objects.all()
        self.additional_values = self.instance.additional_values.all()

        # add fields and init values for the Profile model
        for additional_field in self.additional_fields:

            if additional_field.type == 'text':
                field = forms.CharField(widget=forms.TextInput(attrs={'placeholder': additional_field.text}))
            elif additional_field.type == 'textarea':
                field = forms.CharField(widget=forms.Textarea(attrs={'placeholder': additional_field.text}))
            else:
                raise Exception('Unknown additional_field type.')

            field.label = additional_field.text
            field.help = additional_field.help
            field.required = additional_field.required

            self.fields[additional_field.key] = field

        for additional_field_value in self.additional_values:
            self.fields[additional_field.key].initial = additional_field_value.value

    def save(self, *args, **kwargs):
        super(ProfileForm, self).save(*args, **kwargs)
        self._save_additional_values()

    def _save_additional_values(self, user=None):
        if user is None:
            user = self.instance

        for additional_field in self.additional_fields:
            try:
                additional_value = user.additional_values.get(field=additional_field)
            except AdditionalFieldValue.DoesNotExist:
                additional_value = AdditionalFieldValue(user=user, field=additional_field)

            additional_value.value = self.cleaned_data[additional_field.key]
            additional_value.save()


class SignupForm(ProfileForm):

    use_required_attribute = False

    def __init__(self, *args, **kwargs):
        super(SignupForm, self).__init__(*args, **kwargs)

        # add a consent field, the label is added in the template
        if settings.ACCOUNT_TERMS_OF_USE:
            self.fields['consent'] = forms.BooleanField(required=True)

    def signup(self, request, user):
        self._save_additional_values(user)

        # store the consent field
        if settings.ACCOUNT_TERMS_OF_USE:
            consent = ConsentFieldValue(user=user, consent=self.cleaned_data['consent'])
            consent.save()


class RemoveForm(forms.Form):

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request')
        kwargs.setdefault('label_suffix', '')
        super(RemoveForm, self).__init__(*args, **kwargs)

    email = forms.CharField(widget=forms.TextInput(attrs={'required': 'false'}))
    email.label = _('Email')
    email.widget.attrs = {'class': 'form-control', 'placeholder': email.label}

    password = forms.CharField(widget=forms.PasswordInput)
    password.label = _('Password')
    password.widget.attrs = {'class': 'form-control', 'placeholder': password.label}

    consent = forms.BooleanField(required=True)
    consent.label = _("I confirm that I want my profile to be completely removed. This can not be undone!")



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-01 14:53
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('questions', '0009_meta'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='questionentity',
            name='label_de',
        ),
        migrations.RemoveField(
            model_name='questionentity',
            name='label_en',
        ),
        migrations.RemoveField(
            model_name='section',
            name='label_de',
        ),
        migrations.RemoveField(
            model_name='section',
            name='label_en',
        ),
        migrations.RemoveField(
            model_name='subsection',
            name='label_de',
        ),
        migrations.RemoveField(
            model_name='subsection',
            name='label_en',
        ),
        migrations.AddField(
            model_name='questionentity',
            name='label',
            field=models.CharField(blank=True, help_text='The label to be displayed this question/questionset (auto-generated).', max_length=512, null=True, verbose_name='Label'),
        ),
        migrations.AddField(
            model_name='section',
            name='label',
            field=models.CharField(blank=True, help_text='The label to be displayed this section (auto-generated).', max_length=512, null=True, verbose_name='Label'),
        ),
        migrations.AddField(
            model_name='subsection',
            name='label',
            field=models.CharField(blank=True, help_text='The label to be displayed this subsection (auto-generated).', max_length=512, null=True, verbose_name='Label'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-03 14:31
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0015_label'),
    ]

    operations = []



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-27 14:05
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0030_permissions'),
        ('tasks', '0008_remove_time_period'),
    ]

    operations = [
        migrations.CreateModel(
            name='TimeFrame',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('days_before', models.IntegerField(blank=True, help_text='.', null=True, verbose_name='Days before start date.')),
                ('days_after', models.IntegerField(blank=True, help_text='Days after end date.', null=True, verbose_name='Attribute')),
                ('end_attribute', models.ForeignKey(blank=True, help_text='The Attribute that is setting the end date for this task (optional).', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute', verbose_name='End date attribute')),
                ('start_attribute', models.ForeignKey(blank=True, help_text='The Attribute that is setting the start date for this task.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.Attribute', verbose_name='Start date attribute')),
                ('task', models.OneToOneField(help_text='The task this time frame belongs to.', on_delete=django.db.models.deletion.CASCADE, to='tasks.Task', verbose_name='Task')),
            ],
            options={
                'ordering': ('task',),
                'verbose_name': 'Time frame',
                'verbose_name_plural': 'Time frames',
                'permissions': (('view_timeframe', 'Can view Time frame'),),
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-01-25 11:06
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0004_conditions'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='optionset',
            options={'ordering': ('key',), 'verbose_name': 'OptionSet', 'verbose_name_plural': 'OptionSets'},
        ),
        migrations.RenameField(
            model_name='option',
            old_name='title',
            new_name='key',
        ),
        migrations.RenameField(
            model_name='optionset',
            old_name='title',
            new_name='key',
        ),
    ]



from rdmo.core.renderers import BaseXMLRenderer
from rdmo.core.utils import get_languages


class OptionsRenderer(BaseXMLRenderer):

    def render_optionset(self, xml, optionset):
        xml.startElement('optionset', {'dc:uri': optionset['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, optionset['uri_prefix'])
        self.render_text_element(xml, 'key', {}, optionset['key'])
        self.render_text_element(xml, 'dc:comment', {}, optionset['comment'])
        self.render_text_element(xml, 'order', {}, optionset['order'])
        self.render_text_element(xml, 'provider_key', {}, optionset['provider_key'])
        xml.startElement('conditions', {})
        for condition_uri in optionset['conditions']:
            self.render_text_element(xml, 'condition', {'dc:uri': condition_uri}, None)
        xml.endElement('conditions')
        xml.endElement('optionset')

        if 'options' in optionset and optionset['options']:
            for option in optionset['options']:
                self.render_option(xml, option)

    def render_option(self, xml, option):
        xml.startElement('option', {'dc:uri': option['uri']})
        self.render_text_element(xml, 'uri_prefix', {}, option['uri_prefix'])
        self.render_text_element(xml, 'key', {}, option['key'])
        self.render_text_element(xml, 'path', {}, option['path'])
        self.render_text_element(xml, 'dc:comment', {}, option['comment'])
        self.render_text_element(xml, 'optionset', {'dc:uri': option['optionset']}, None)
        self.render_text_element(xml, 'order', {}, option['order'])

        for lang_code, lang_string, lang_field in get_languages():
            self.render_text_element(xml, 'text', {'lang': lang_code}, option['text_%s' % lang_code])

        self.render_text_element(xml, 'additional_input', {}, option['additional_input'])
        xml.endElement('option')


class OptionSetRenderer(OptionsRenderer):

    def render_document(self, xml, optionsets):
        xml.startElement('rdmo', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        for optionset in optionsets:
            self.render_optionset(xml, optionset)
        xml.endElement('rdmo')


class OptionRenderer(OptionsRenderer):

    def render_document(self, xml, options):
        xml.startElement('rdmo', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        for option in options:
            self.render_option(xml, option)
        xml.endElement('rdmo')



from rest_framework import serializers

from rdmo.core.serializers import TranslationSerializerMixin

from ..models import View


class ViewExportSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    catalogs = serializers.SerializerMethodField()

    class Meta:
        model = View
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'catalogs',
            'template'
        )
        trans_fields = (
            'title',
            'help'
        )

    def get_catalogs(self, obj):
        return [catalog.uri for catalog in obj.catalogs.all()]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-02-04 13:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0028_path'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attributeentity',
            name='path',
            field=models.CharField(db_index=True, help_text='The path part of the URI of this attribute/entity (auto-generated).', max_length=512, verbose_name='Path'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-20 08:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0021_data_migrations'),
    ]

    operations = [
        migrations.AlterField(
            model_name='option',
            name='optionset',
            field=models.ForeignKey(help_text='The option set this option belongs to.', on_delete=django.db.models.deletion.CASCADE, related_name='options', to='options.OptionSet', verbose_name='Option set'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-08-03 12:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('conditions', '0003_meta'),
    ]

    operations = [
        migrations.AddField(
            model_name='condition',
            name='title',
            field=models.CharField(default='', max_length=256),
            preserve_default=False,
        ),
    ]



from rest_framework import serializers

from ..models import Attribute


class AttributeExportSerializer(serializers.ModelSerializer):

    parent = serializers.CharField(source='parent.uri', default=None, read_only=True)

    class Meta:
        model = Attribute
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'path',
            'comment',
            'parent'
        )



from django.urls import include, path
from rest_framework import routers

from ..viewsets import OptionSetViewSet, OptionViewSet, ProviderViewSet

app_name = 'v1-options'

router = routers.DefaultRouter()
router.register(r'optionsets', OptionSetViewSet, basename='optionset')
router.register(r'options', OptionViewSet, basename='option')
router.register(r'providers', ProviderViewSet, basename='provider')

urlpatterns = [
    path('', include(router.urls)),
]



from rdmo.core.renderers import BaseXMLRenderer


class XMLRenderer(BaseXMLRenderer):

    def render_document(self, xml, project):
        xml.startElement('project', {
            'xmlns:dc': 'http://purl.org/dc/elements/1.1/',
            'created': self.created
        })
        self.render_text_element(xml, 'title', {}, project['title'])
        self.render_text_element(xml, 'description', {}, project['description'])
        self.render_text_element(xml, 'catalog', {'dc:uri': project['catalog']}, None)

        if 'tasks' in project and project['tasks']:
            xml.startElement('tasks', {})
            for task in project['tasks']:
                self.render_text_element(xml, 'task', {'dc:uri': task}, None)
            xml.endElement('tasks')

        if 'views' in project and project['views']:
            xml.startElement('views', {})
            for view in project['views']:
                self.render_text_element(xml, 'view', {'dc:uri': view}, None)
            xml.endElement('views')

        if 'snapshots' in project and project['snapshots']:
            xml.startElement('snapshots', {})
            for snapshot in project['snapshots']:
                self.render_snapshot(xml, snapshot)
            xml.endElement('snapshots')

        if 'values' in project and project['values']:
            xml.startElement('values', {})
            for value in project['values']:
                self.render_value(xml, value)
            xml.endElement('values')

        self.render_text_element(xml, 'created', {}, project['created'])
        self.render_text_element(xml, 'updated', {}, project['updated'])
        xml.endElement('project')

    def render_snapshot(self, xml, snapshot):
        xml.startElement('snapshot', {})
        self.render_text_element(xml, 'title', {}, snapshot['title'])
        self.render_text_element(xml, 'description', {}, snapshot['description'])

        if 'values' in snapshot and snapshot['values']:
            xml.startElement('values', {})
            for value in snapshot['values']:
                self.render_value(xml, value)
            xml.endElement('values')

        self.render_text_element(xml, 'created', {}, snapshot['created'])
        self.render_text_element(xml, 'updated', {}, snapshot['updated'])
        xml.endElement('snapshot')

    def render_value(self, xml, value):
        xml.startElement('value', {})
        self.render_text_element(xml, 'attribute', {'dc:uri': value['attribute']}, None)
        self.render_text_element(xml, 'set_prefix', {}, value['set_prefix'])
        self.render_text_element(xml, 'set_index', {}, value['set_index'])
        self.render_text_element(xml, 'collection_index', {}, value['collection_index'])
        self.render_text_element(xml, 'text', {}, value['text'])
        self.render_text_element(xml, 'option', {'dc:uri': value['option']}, None)
        self.render_text_element(xml, 'file', {'name': value['file_name']}, value['file_content'])
        self.render_text_element(xml, 'value_type', {}, value['value_type'])
        self.render_text_element(xml, 'unit', {}, value['unit'])
        self.render_text_element(xml, 'external_id', {}, value['external_id'])
        self.render_text_element(xml, 'created', {}, value['created'])
        self.render_text_element(xml, 'updated', {}, value['updated'])
        xml.endElement('value')



from ..models import Option, OptionSet


def test_optionset_str(db):
    instances = OptionSet.objects.all()
    for instance in instances:
        assert str(instance)


def test_optionset_clean(db):
    instances = OptionSet.objects.all()
    for instance in instances:
        instance.clean()


def test_options_str(db):
    instances = Option.objects.all()
    for instance in instances:
        assert str(instance)


def test_optionset_copy(db):
    instances = OptionSet.objects.all()
    for instance in instances:
        new_uri_prefix = instance.uri_prefix + '-'
        new_key = instance.key + '-'
        new_instance = instance.copy(new_uri_prefix, new_key)
        assert new_instance.uri_prefix == new_uri_prefix
        assert new_instance.key == new_key
        assert list(new_instance.conditions.values('id')) == list(new_instance.conditions.values('id'))
        assert new_instance.options.count() == instance.options.count()


def test_options_clean(db):
    instances = Option.objects.all()
    for instance in instances:
        instance.clean()


def test_options_copy(db):
    instances = Option.objects.all()
    for instance in instances:
        new_uri_prefix = instance.uri_prefix + '-'
        new_key = instance.key + '-'
        new_instance = instance.copy(new_uri_prefix, new_key)
        assert new_instance.uri_prefix == new_uri_prefix
        assert new_instance.key == new_key



from django_filters import CharFilter, FilterSet
from rest_framework.filters import BaseFilterBackend

from .models import Project


class ProjectFilter(FilterSet):
    title = CharFilter(field_name='title', lookup_expr='icontains')

    class Meta:
        model = Project
        fields = ('title', )


class SnapshotFilterBackend(BaseFilterBackend):

    def filter_queryset(self, request, queryset, view):
        if view.detail:
            return queryset

        snapshot = request.GET.get('snapshot')
        if snapshot:
            try:
                snapshot_pk = int(snapshot)
            except (ValueError, TypeError):
                snapshot_pk = None

            queryset = queryset.filter(snapshot__pk=snapshot_pk)
        else:
            queryset = queryset.filter(snapshot=None)

        return queryset


class ValueFilterBackend(BaseFilterBackend):

    def filter_queryset(self, request, queryset, view):
        if view.detail:
            return queryset

        attributes = [int(attribute) for attribute in request.GET.getlist('attribute') if attribute.isdigit()]
        if attributes:
            queryset = queryset.filter(attribute__in=attributes)

        return queryset



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-21 13:12
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('domain', '0038_rename_attributeentity_to_attribute'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='attribute',
            options={'ordering': ('uri',), 'permissions': (('view_attribute', 'Can view Attribute'),), 'verbose_name': 'Attribute', 'verbose_name_plural': 'Attributes'},
        ),
        migrations.RemoveField(
            model_name='attribute',
            name='is_attribute',
        ),
        migrations.AlterField(
            model_name='attribute',
            name='comment',
            field=models.TextField(blank=True, help_text='Additional information about this attribute.', null=True, verbose_name='Comment'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='key',
            field=models.SlugField(blank=True, help_text='The internal identifier of this attribute.', max_length=128, null=True, verbose_name='Key'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='parent',
            field=mptt.fields.TreeForeignKey(blank=True, help_text='Parent attribute in the domain model.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='domain.Attribute', verbose_name='Parent attribute'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='path',
            field=models.CharField(db_index=True, help_text='The path part of the URI of this attribute (auto-generated).', max_length=512, verbose_name='Path'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='uri',
            field=models.URLField(blank=True, help_text='The Uniform Resource Identifier of this attribute set (auto-generated).', max_length=640, null=True, verbose_name='URI'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='uri_prefix',
            field=models.URLField(blank=True, help_text='The prefix for the URI of this attribute.', max_length=256, null=True, verbose_name='URI Prefix'),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-05 11:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('views', '0021_view_catalogs'),
        ('projects', '0029_remove_manager'),
    ]

    operations = [
        migrations.AddField(
            model_name='project',
            name='views',
            field=models.ManyToManyField(blank=True, help_text='The views that will be used for this project.', to='views.View', verbose_name='Views'),
        ),
    ]



from django import forms
from django.contrib import admin

from rdmo.core.utils import get_language_fields

from .models import Option, OptionSet
from .validators import (OptionLockedValidator, OptionSetLockedValidator,
                         OptionSetUniqueURIValidator, OptionUniqueURIValidator)


class OptionSetAdminForm(forms.ModelForm):
    key = forms.SlugField(required=True)

    class Meta:
        model = OptionSet
        fields = '__all__'

    def clean(self):
        OptionSetUniqueURIValidator(self.instance)(self.cleaned_data)
        OptionSetLockedValidator(self.instance)(self.cleaned_data)


class OptionAdminForm(forms.ModelForm):
    key = forms.SlugField(required=True)

    class Meta:
        model = Option
        fields = '__all__'

    def clean(self):
        OptionUniqueURIValidator(self.instance)(self.cleaned_data)
        OptionLockedValidator(self.instance)(self.cleaned_data)


class OptionSetAdmin(admin.ModelAdmin):
    form = OptionSetAdminForm

    search_fields = ('uri', )
    list_display = ('uri', )
    readonly_fields = ('uri', )


class OptionAdmin(admin.ModelAdmin):
    form = OptionAdminForm

    search_fields = ['uri'] + get_language_fields('text')
    list_display = ('uri', 'text', 'additional_input')
    readonly_fields = ('uri', 'path')
    list_filter = ('optionset', 'additional_input')


admin.site.register(OptionSet, OptionSetAdmin)
admin.site.register(Option, OptionAdmin)



from rdmo.core.serializers import TranslationSerializerMixin
from rest_framework import serializers

from ..models import Option, OptionSet


class OptionExportSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    optionset = serializers.CharField(source='optionset.uri', default=None, read_only=True)

    class Meta:
        model = Option
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'path',
            'comment',
            'order',
            'additional_input',
            'optionset'
        )
        trans_fields = (
            'text',
        )


class OptionSetExportSerializer(serializers.ModelSerializer):

    options = OptionExportSerializer(many=True)
    conditions = serializers.SerializerMethodField()

    class Meta:
        model = OptionSet
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'order',
            'provider_key',
            'options',
            'conditions'
        )

    def get_conditions(self, obj):
        return [condition.uri for condition in obj.conditions.all()]



import logging

from django.http import Http404
from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import CreateView, DeleteView, UpdateView, View

from rdmo.core.plugins import get_plugin
from rdmo.core.views import ObjectPermissionMixin, RedirectViewMixin

from ..forms import IntegrationForm
from ..models import Integration, Project

logger = logging.getLogger(__name__)


class IntegrationCreateView(ObjectPermissionMixin, RedirectViewMixin, CreateView):
    model = Integration
    form_class = IntegrationForm
    permission_required = 'projects.add_integration_object'

    def dispatch(self, *args, **kwargs):
        self.project = get_object_or_404(Project.objects.all(), pk=self.kwargs['project_id'])
        self.provider_key = self.kwargs['provider_key']
        return super().dispatch(*args, **kwargs)

    def get_permission_object(self):
        return self.project

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['project'] = self.project
        kwargs['provider_key'] = self.provider_key
        return kwargs

    def get_context_data(self, **kwargs):
        kwargs['provider'] = get_plugin('SERVICE_PROVIDERS', self.provider_key)
        return super().get_context_data(**kwargs)

    def get_success_url(self):
        return self.project.get_absolute_url()


class IntegrationUpdateView(ObjectPermissionMixin, RedirectViewMixin, UpdateView):
    form_class = IntegrationForm
    permission_required = 'projects.change_integration_object'

    def get_queryset(self):
        return Integration.objects.filter(project_id=self.kwargs.get('project_id'))

    def get_permission_object(self):
        return self.get_object().project

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['project'] = self.get_object().project
        return kwargs

    def get_context_data(self, **kwargs):
        kwargs['provider'] = self.get_object().provider
        return super().get_context_data(**kwargs)

    def get_success_url(self):
        return self.get_object().project.get_absolute_url()


class IntegrationDeleteView(ObjectPermissionMixin, RedirectViewMixin, DeleteView):
    permission_required = 'projects.delete_integration_object'

    def get_queryset(self):
        return Integration.objects.filter(project_id=self.kwargs.get('project_id'))

    def get_permission_object(self):
        return self.get_object().project

    def get_success_url(self):
        return self.get_object().project.get_absolute_url()


@method_decorator(csrf_exempt, name='dispatch')
class IntegrationWebhookView(View):

    def post(self, request, *args, **kwargs):
        pk = kwargs.get('pk')
        project_id = kwargs.get('project_id')

        try:
            integration = Integration.objects.filter(project_id=project_id).get(pk=pk)
            return integration.provider.webhook(request, integration)
        except Integration.DoesNotExist:
            raise Http404



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-10-24 14:53
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='option',
            name='optionset',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='options', to='options.OptionSet'),
        ),
        migrations.AlterField(
            model_name='option',
            name='order',
            field=models.IntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='optionset',
            name='order',
            field=models.IntegerField(default=0),
        ),
    ]



import pytest
from django.conf import settings
from django.core.exceptions import ValidationError
from rest_framework.exceptions import \
    ValidationError as RestFameworkValidationError

from ..models import View
from ..serializers.v1 import ViewSerializer
from ..validators import ViewUniqueURIValidator


def test_unique_uri_validator_create(db):
    ViewUniqueURIValidator()({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_unique_uri_validator_create_error(db):
    with pytest.raises(ValidationError):
        ViewUniqueURIValidator()({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': View.objects.last().key
        })


def test_unique_uri_validator_update(db):
    view = View.objects.first()

    ViewUniqueURIValidator(view)({
        'uri_prefix': view.uri_prefix,
        'key': view.key
    })


def test_unique_uri_validator_update_error(db):
    view = View.objects.first()

    with pytest.raises(ValidationError):
        ViewUniqueURIValidator(view)({
            'uri_prefix': view.uri_prefix,
            'key': View.objects.last().key
        })


def test_unique_uri_validator_serializer_create(db):
    validator = ViewUniqueURIValidator()
    validator.set_context(ViewSerializer())

    validator({
        'uri_prefix': settings.DEFAULT_URI_PREFIX,
        'key': 'test'
    })


def test_unique_uri_validator_serializer_create_error(db):
    validator = ViewUniqueURIValidator()
    validator.set_context(ViewSerializer())

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': settings.DEFAULT_URI_PREFIX,
            'key': View.objects.last().key
        })


def test_unique_uri_validator_serializer_update(db):
    view = View.objects.first()

    validator = ViewUniqueURIValidator()
    validator.set_context(ViewSerializer(instance=view))

    validator({
        'uri_prefix': view.uri_prefix,
        'key': view.key
    })


def test_unique_uri_validator_serializer_update_error(db):
    view = View.objects.first()

    validator = ViewUniqueURIValidator()
    validator.set_context(ViewSerializer(instance=view))

    with pytest.raises(RestFameworkValidationError):
        validator({
            'uri_prefix': view.uri_prefix,
            'key': View.objects.last().key
        })



# Generated by Django 2.2.16 on 2021-01-22 19:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0032_task_catalogs'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='locked',
            field=models.BooleanField(default=False, help_text='Designates whether this task can be changed.', verbose_name='Locked'),
        ),
    ]



from rest_framework import serializers
from rest_framework.reverse import reverse

from rdmo.core.serializers import (MarkdownSerializerMixin, SiteSerializer,
                                   TranslationSerializerMixin)
from rdmo.core.utils import get_language_warning

from ..models import Task
from ..validators import TaskLockedValidator, TaskUniqueURIValidator


class TaskSerializer(TranslationSerializerMixin, serializers.ModelSerializer):

    key = serializers.SlugField(required=True)

    class Meta:
        model = Task
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'locked',
            'available',
            'catalogs',
            'sites',
            'groups',
            'start_attribute',
            'end_attribute',
            'days_before',
            'days_after',
            'conditions'
        )
        trans_fields = (
            'title',
            'text'
        )
        validators = (
            TaskUniqueURIValidator(),
            TaskLockedValidator()
        )


class TaskIndexSerializer(MarkdownSerializerMixin, serializers.ModelSerializer):

    markdown_fields = ('text', )

    sites = SiteSerializer(many=True, read_only=True)
    warning = serializers.SerializerMethodField()
    xml_url = serializers.SerializerMethodField()

    class Meta:
        model = Task
        fields = (
            'id',
            'uri',
            'uri_prefix',
            'key',
            'locked',
            'available',
            'sites',
            'title',
            'text',
            'warning',
            'xml_url'
        )

    def get_warning(self, obj):
        return get_language_warning(obj, 'title') or get_language_warning(obj, 'text')

    def get_xml_url(self, obj):
        return reverse('v1-tasks:task-detail-export', args=[obj.pk])



from rest_framework import serializers

from ..models import Condition


class ConditionExportSerializer(serializers.ModelSerializer):

    source = serializers.CharField(source='source.uri', default=None, read_only=True)
    target_option = serializers.CharField(source='target_option.uri', default=None, read_only=True)

    class Meta:
        model = Condition
        fields = (
            'uri',
            'uri_prefix',
            'key',
            'comment',
            'source',
            'relation',
            'target_text',
            'target_option'
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-03-27 12:47
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0007_permissions'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='task',
            name='attribute',
        ),
        migrations.RemoveField(
            model_name='task',
            name='time_period',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-03 15:55
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    atomic = False

    dependencies = [
        ('questions', '0027_remove_question_entity_and_question'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='QuestionItem',
            new_name='Question',
        ),
    ]



import rules


@rules.predicate
def is_project_member(user, project):
    return user in project.member or (project.parent and is_project_member(user, project.parent))


@rules.predicate
def is_current_project_member(user, project):
    return user in project.member


@rules.predicate
def is_project_owner(user, project):
    return user in project.owners or (project.parent and is_project_owner(user, project.parent))


@rules.predicate
def is_project_manager(user, project):
    return user in project.managers or (project.parent and is_project_manager(user, project.parent))


@rules.predicate
def is_project_author(user, project):
    return user in project.authors or (project.parent and is_project_author(user, project.parent))


@rules.predicate
def is_project_guest(user, project):
    return user in project.guests or (project.parent and is_project_guest(user, project.parent))


@rules.predicate
def is_site_manager(user, project):
    if user.is_authenticated:
        return user.role.manager.filter(pk=project.site.pk).exists()
    else:
        return False


rules.add_perm('projects.view_project_object', is_project_member | is_site_manager)
rules.add_perm('projects.change_project_object', is_project_manager | is_project_owner | is_site_manager)
rules.add_perm('projects.delete_project_object', is_project_owner | is_site_manager)
rules.add_perm('projects.leave_project_object', is_current_project_member)
rules.add_perm('projects.export_project_object', is_project_owner | is_project_manager | is_site_manager)
rules.add_perm('projects.import_project_object', is_project_owner | is_project_manager | is_site_manager)

rules.add_perm('projects.view_membership_object', is_project_member | is_site_manager)
rules.add_perm('projects.add_membership_object', is_project_owner | is_site_manager)
rules.add_perm('projects.change_membership_object', is_project_owner | is_site_manager)
rules.add_perm('projects.delete_membership_object', is_project_owner | is_site_manager)

rules.add_perm('projects.view_invite_object', is_project_owner | is_site_manager)
rules.add_perm('projects.add_invite_object', is_project_owner | is_site_manager)
rules.add_perm('projects.change_invite_object', is_project_owner | is_site_manager)
rules.add_perm('projects.delete_invite_object', is_project_owner | is_site_manager)

rules.add_perm('projects.view_integration_object', is_project_member | is_site_manager)
rules.add_perm('projects.add_integration_object', is_project_owner | is_project_manager | is_site_manager)
rules.add_perm('projects.change_integration_object', is_project_owner | is_project_manager | is_site_manager)
rules.add_perm('projects.delete_integration_object', is_project_owner | is_project_manager | is_site_manager)

rules.add_perm('projects.view_issue_object', is_project_member | is_site_manager)
rules.add_perm('projects.add_issue_object', is_project_manager | is_project_owner | is_site_manager)
rules.add_perm('projects.change_issue_object', is_project_author | is_project_manager | is_project_owner | is_site_manager)
rules.add_perm('projects.delete_issue_object', is_project_manager | is_project_owner | is_site_manager)

rules.add_perm('projects.view_snapshot_object', is_project_member | is_site_manager)
rules.add_perm('projects.add_snapshot_object', is_project_manager | is_project_owner | is_site_manager)
rules.add_perm('projects.change_snapshot_object', is_project_manager | is_project_owner | is_site_manager)
rules.add_perm('projects.rollback_snapshot_object', is_project_manager | is_project_owner | is_site_manager)

rules.add_perm('projects.view_value_object', is_project_member | is_site_manager)
rules.add_perm('projects.add_value_object', is_project_author | is_project_manager | is_project_owner | is_site_manager)
rules.add_perm('projects.change_value_object', is_project_author | is_project_manager | is_project_owner | is_site_manager)
rules.add_perm('projects.delete_value_object', is_project_author | is_project_manager | is_project_owner | is_site_manager)

rules.add_perm('questions.view_questionset_object', is_project_member | is_site_manager)

# TODO: use one of the permissions above
rules.add_perm('projects.is_project_owner', is_project_owner)



from django.conf import settings


def get_overlays(url_name):
    overlays = settings.OVERLAYS.get(url_name)[:]

    if not settings.PROJECT_ISSUES and 'project-issues' in overlays:
        overlays.remove('project-issues')

    if not settings.PROJECT_VIEWS and 'project-views' in overlays:
        overlays.remove('project-views')

    if not settings.PROJECT_EXPORTS and 'export-project' in overlays:
        overlays.remove('export-project')

    if not settings.PROJECT_IMPORTS and 'import-project' in overlays:
        overlays.remove('import-project')

    return overlays



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-05 14:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('options', '0012_meta'),
        ('questions', '0016_data_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='question',
            name='optionsets',
            field=models.ManyToManyField(blank=True, help_text='Option sets for this question.', to='options.OptionSet', verbose_name='Option sets'),
        ),
    ]



import logging

from django.conf import settings
from django.utils.translation import gettext_lazy as _
from django.views.generic import ListView, TemplateView

from rdmo.core.exports import XMLResponse
from rdmo.core.utils import get_model_field_meta, render_to_format
from rdmo.core.views import CSRFViewMixin, ModelPermissionMixin

from .models import Condition
from .renderers import ConditionRenderer
from .serializers.export import ConditionExportSerializer

log = logging.getLogger(__name__)


class ConditionsView(ModelPermissionMixin, CSRFViewMixin, TemplateView):
    template_name = 'conditions/conditions.html'
    permission_required = 'conditions.view_condition'

    def get_context_data(self, **kwargs):
        context = super(ConditionsView, self).get_context_data(**kwargs)
        context['export_formats'] = settings.EXPORT_FORMATS
        context['meta'] = {
            'Condition': get_model_field_meta(Condition)
        }
        return context


class ConditionsExportView(ModelPermissionMixin, ListView):
    model = Condition
    context_object_name = 'conditions'
    permission_required = 'conditions.view_condition'

    def render_to_response(self, context, **response_kwargs):
        format = self.kwargs.get('format')
        if format == 'xml':
            serializer = ConditionExportSerializer(context['conditions'], many=True)
            xml = ConditionRenderer().render(serializer.data)
            return XMLResponse(xml, name='conditions')
        else:
            return render_to_format(self.request, format, _('Conditions'), 'conditions/conditions_export.html', context)



from xml.dom.minidom import parseString

from django.http import HttpResponse


class XMLResponse(HttpResponse):

    def __init__(self, xml, name=None):
        super().__init__(prettify_xml(xml), content_type='application/xml')
        if name:
            self['Content-Disposition'] = 'filename="{}.xml"'.format(name)


def prettify_xml(xmlstring):
    xmlobj = parseString(xmlstring)
    return xmlobj.toprettyxml(
        indent='\t', newl='\n', encoding='UTF-8'
    )



from django.utils.translation import gettext_lazy as _

VALUE_TYPE_TEXT = 'text'
VALUE_TYPE_URL = 'url'
VALUE_TYPE_INTEGER = 'integer'
VALUE_TYPE_FLOAT = 'float'
VALUE_TYPE_BOOLEAN = 'boolean'
VALUE_TYPE_DATETIME = 'datetime'
VALUE_TYPE_OPTIONS = 'option'
VALUE_TYPE_FILE = 'file'
VALUE_TYPE_CHOICES = (
    (VALUE_TYPE_TEXT, _('Text')),
    (VALUE_TYPE_URL, _('URL')),
    (VALUE_TYPE_INTEGER, _('Integer')),
    (VALUE_TYPE_FLOAT, _('Float')),
    (VALUE_TYPE_BOOLEAN, _('Boolean')),
    (VALUE_TYPE_DATETIME, _('Datetime')),
    (VALUE_TYPE_OPTIONS, _('Option')),
    (VALUE_TYPE_FILE, _('File'))
)

PERMISSIONS = {
    'condition': (
        'conditions.add_condition', 'conditions.change_condition', 'conditions.delete_condition'
    ),
    'attribute': (
        'domain.add_attribute', 'domain.change_attribute', 'domain.delete_attribute'
    ),
    'optionset': (
        'options.add_optionset', 'options.change_optionset', 'options.delete_optionset'
    ),
    'option': (
        'options.add_option', 'options.change_option', 'options.delete_option'
    ),
    'catalog': (
        'questions.add_catalog', 'questions.change_catalog', 'questions.delete_catalog'
    ),
    'section': (
        'questions.add_section', 'questions.change_section', 'questions.delete_section'
    ),
    'questionset': (
        'questions.add_questionset', 'questions.change_questionset', 'questions.delete_questionset'
    ),
    'question': (
        'questions.add_question', 'questions.change_question', 'questions.delete_question'
    ),
    'task': (
        'tasks.add_task', 'tasks.change_task', 'tasks.delete_task'
    ),
    'view': (
        'views.add_view', 'views.change_view', 'views.delete_view'
    )
}



# Generated by Django 2.2rc1 on 2019-03-26 13:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0025_remove_null_true'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='membership',
            options={'ordering': ('project__title',), 'verbose_name': 'Membership', 'verbose_name_plural': 'Memberships'},
        ),
        migrations.AlterModelOptions(
            name='project',
            options={'ordering': ('title',), 'verbose_name': 'Project', 'verbose_name_plural': 'Projects'},
        ),
        migrations.AlterModelOptions(
            name='snapshot',
            options={'ordering': ('project', '-created'), 'verbose_name': 'Snapshot', 'verbose_name_plural': 'Snapshots'},
        ),
        migrations.AlterModelOptions(
            name='value',
            options={'ordering': ('attribute', 'set_index', 'collection_index'), 'verbose_name': 'Value', 'verbose_name_plural': 'Values'},
        ),
        migrations.AlterField(
            model_name='snapshot',
            name='project',
            field=models.ForeignKey(help_text='The project this snapshot belongs to.', null=True, on_delete=django.db.models.deletion.CASCADE, related_name='snapshots', to='projects.Project', verbose_name='Project'),
        ),
    ]



import re

import pytest
from django.urls import reverse

from rdmo.views.models import View

from ..models import Project

users = (
    ('owner', 'owner'),
    ('manager', 'manager'),
    ('author', 'author'),
    ('guest', 'guest'),
    ('user', 'user'),
    ('site', 'site'),
    ('anonymous', None),
)

view_project_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5, 7],
    'author': [1, 3, 5, 8],
    'guest': [1, 3, 5, 9],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5, 6, 7, 8, 9]
}

change_project_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5, 7],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5, 6, 7, 8, 9]
}

delete_project_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5, 6, 7, 8, 9]
}

export_project_permission_map = {
    'owner': [1, 2, 3, 4, 5],
    'manager': [1, 3, 5, 7],
    'api': [1, 2, 3, 4, 5],
    'site': [1, 2, 3, 4, 5, 6, 7, 8, 9]
}

projects = [1, 2, 3, 4, 5, 6, 7, 8, 9]

export_formats = ('rtf', 'odt', 'docx', 'html', 'markdown', 'tex', 'pdf')

site_id = 1
parent_project_id = 1
catalog_id = 1


@pytest.mark.parametrize('username,password', users)
def test_list(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('projects')
    response = client.get(url)

    projects = re.findall(r'/projects/(\d+)/', response.content.decode())

    if password:
        assert response.status_code == 200

        if username == 'site':
            assert projects == []
        else:
            assert sorted(list(set([int(project_id) for project_id in projects]))) \
                == view_project_permission_map.get(username, [])
    else:
        assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
def test_site(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('site_projects')
    response = client.get(url)

    projects = re.findall(r'/projects/(\d+)/update/', response.content.decode())

    if username == 'site':
        assert sorted([int(project_id) for project_id in projects]) \
            == view_project_permission_map.get(username, [])
    elif password:
        assert response.status_code == 403
    else:
        assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_detail(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project', args=[project_id])
    response = client.get(url)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
def test_project_create_get(db, client, username, password):
    client.login(username=username, password=password)

    url = reverse('project_create')
    response = client.get(url)

    if password:
        assert response.status_code == 200

        # check the parent select dropdown
        for project_id in re.findall(r'<option value="(\d+)"', response.content.decode()):
            assert int(project_id) in view_project_permission_map.get(username, [])
    else:
        assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
def test_project_create_post(db, client, username, password):
    client.login(username=username, password=password)
    project_count = Project.objects.count()

    url = reverse('project_create')
    data = {
        'title': 'A new project',
        'description': 'Some description',
        'catalog': catalog_id
    }
    response = client.post(url, data)

    if password:
        assert response.status_code == 302
        assert Project.objects.count() == project_count + 1
    else:
        assert response.status_code == 302
        assert Project.objects.count() == project_count


@pytest.mark.parametrize('username,password', users)
def test_project_create_parent_post(db, client, username, password):
    client.login(username=username, password=password)
    project_count = Project.objects.count()

    url = reverse('project_create')
    data = {
        'title': 'A new project',
        'description': 'Some description',
        'catalog': catalog_id,
        'parent': parent_project_id
    }
    response = client.post(url, data)

    if username == 'user':
        assert response.status_code == 200
        assert Project.objects.count() == project_count
    elif password:
        assert response.status_code == 302
        assert Project.objects.count() == project_count + 1
    else:
        assert response.status_code == 302
        assert Project.objects.count() == project_count


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_update', args=[project_id])
    response = client.get(url)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_post(db, client, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    url = reverse('project_update', args=[project_id])
    data = {
        'title': 'New title',
        'description': project.description,
        'catalog': project.catalog.pk
    }
    response = client.post(url, data)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 302
        assert Project.objects.get(pk=project_id).title == 'New title'
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert Project.objects.get(pk=project_id).title == project.title


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_post_parent(db, client, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    url = reverse('project_update', args=[project_id])
    data = {
        'title': project.title,
        'description': project.description,
        'catalog': project.catalog.pk,
        'parent': parent_project_id
    }
    response = client.post(url, data)

    if project_id in change_project_permission_map.get(username, []):
        if project_id == parent_project_id:
            assert response.status_code == 200
            assert Project.objects.get(pk=project_id).parent == project.parent
        else:
            assert response.status_code == 302
            assert Project.objects.get(pk=project_id).parent_id == parent_project_id
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert Project.objects.get(pk=project_id).parent == project.parent


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_information_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_update_information', args=[project_id])
    response = client.get(url)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_information_post(db, client, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    url = reverse('project_update_information', args=[project_id])
    data = {
        'title': 'Lorem ipsum dolor sit amet',
        'description': 'At vero eos et accusam et justo duo dolores et ea rebum.'
    }
    response = client.post(url, data)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 302
        assert Project.objects.get(pk=project_id).title == 'Lorem ipsum dolor sit amet'
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert Project.objects.get(pk=project_id).title == project.title


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_catalog_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_update_catalog', args=[project_id])
    response = client.get(url)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_catalog_post(db, client, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    url = reverse('project_update_catalog', args=[project_id])
    data = {
        'catalog': catalog_id
    }
    response = client.post(url, data)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 302
        assert Project.objects.get(pk=project_id).catalog_id == catalog_id
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert Project.objects.get(pk=project_id).catalog == project.catalog


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_tasks_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_update_tasks', args=[project_id])
    response = client.get(url)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_tasks_post(db, client, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    url = reverse('project_update_tasks', args=[project_id])
    data = {
        'tasks': []
    }
    response = client.post(url, data)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 302
        assert list(Project.objects.get(pk=project_id).tasks.values('id')) == []
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert list(Project.objects.get(pk=project_id).tasks.values('id')) == list(project.tasks.values('id'))


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_views_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_update_views', args=[project_id])
    response = client.get(url)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_views_post(db, client, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    url = reverse('project_update_views', args=[project_id])
    data = {
        'views': []
    }
    response = client.post(url, data)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 302
        assert list(Project.objects.get(pk=project_id).views.values('id')) == []
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert list(Project.objects.get(pk=project_id).views.values('id')) == list(project.views.values('id'))


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_parent_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_update_parent', args=[project_id])
    response = client.get(url)

    if project_id in change_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_update_parent_post(db, client, username, password, project_id):
    client.login(username=username, password=password)
    project = Project.objects.get(pk=project_id)

    url = reverse('project_update_parent', args=[project_id])
    data = {
        'parent': parent_project_id
    }
    response = client.post(url, data)

    if project_id in change_project_permission_map.get(username, []):
        if project_id == parent_project_id:
            assert response.status_code == 200
            assert Project.objects.get(pk=project_id).parent == project.parent
        else:
            assert response.status_code == 302
            assert Project.objects.get(pk=project_id).parent_id == parent_project_id
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert Project.objects.get(pk=project_id).parent == project.parent


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_delete_get(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_delete', args=[project_id])
    response = client.get(url)

    if project_id in delete_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_delete_post(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_delete', args=[project_id])
    response = client.post(url)

    if project_id in delete_project_permission_map.get(username, []):
        assert response.status_code == 302
        assert not Project.objects.filter(pk=project_id).first()
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302

        assert Project.objects.filter(pk=project_id).first()


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_export_xml(db, client, files, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_export', args=[project_id, 'xml'])
    response = client.get(url)

    if project_id in export_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_export_csv(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_export', args=[project_id, 'csvcomma'])
    response = client.get(url)

    if project_id in export_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_export_csvsemicolon(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_export', args=[project_id, 'csvsemicolon'])
    response = client.get(url)

    if project_id in export_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_answers(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_answers', args=[project_id])
    response = client.get(url)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('export_format', export_formats)
def test_project_answers_export(db, client, username, password, project_id, export_format):
    client.login(username=username, password=password)

    url = reverse('project_answers_export', args=[project_id, export_format])
    response = client.get(url)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 200
    else:
        if password:
            assert response.status_code == 403
        else:
            assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_view(db, client, username, password, project_id):
    client.login(username=username, password=password)
    project_views = Project.objects.get(pk=project_id).views.all()

    for view in View.objects.all():
        url = reverse('project_view', args=[project_id, view.id])
        response = client.get(url)

        if project_id in view_project_permission_map.get(username, []):
            if view in project_views:
                assert response.status_code == 200
            else:
                assert response.status_code == 404
        else:
            if password:
                assert response.status_code == 403
            else:
                assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
@pytest.mark.parametrize('export_format', export_formats)
def test_project_view_export(db, client, username, password, project_id, export_format):
    client.login(username=username, password=password)
    project_views = Project.objects.get(pk=project_id).views.all()

    for view in View.objects.all():
        url = reverse('project_view_export', args=[project_id, view.pk, export_format])
        response = client.get(url)

        if project_id in view_project_permission_map.get(username, []):
            if view in project_views:
                assert response.status_code == 200
            else:
                assert response.status_code == 404
        else:
            if password:
                assert response.status_code == 403
            else:
                assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_questions(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_questions', args=[project_id])
    response = client.get(url)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 200
    elif password:
        assert response.status_code == 403
    else:
        assert response.status_code == 302


@pytest.mark.parametrize('username,password', users)
@pytest.mark.parametrize('project_id', projects)
def test_project_error(db, client, username, password, project_id):
    client.login(username=username, password=password)

    url = reverse('project_questions', args=[project_id])
    response = client.get(url)

    if project_id in view_project_permission_map.get(username, []):
        assert response.status_code == 200
    elif password:
        assert response.status_code == 403
    else:
        assert response.status_code == 302



# Generated by Django 2.2.13 on 2020-06-26 14:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('projects', '0032_data_migration'),
    ]

    operations = [
        migrations.AlterField(
            model_name='value',
            name='value_type',
            field=models.CharField(choices=[('text', 'Text'), ('url', 'URL'), ('integer', 'Integer'), ('float', 'Float'), ('boolean', 'Boolean'), ('datetime', 'Datetime'), ('option', 'Option')], default='text', help_text='Type of this value.', max_length=8, verbose_name='Value type'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-08-21 13:00
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tasks', '0011_task_text'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timeframe',
            name='end_attribute',
            field=models.ForeignKey(blank=True, help_text='The Attribute that is setting the end date for this task (optional, if no end date attribute is given, the start date attribute sets also the end date).', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.AttributeEntity', verbose_name='End date attribute'),
        ),
        migrations.AlterField(
            model_name='timeframe',
            name='start_attribute',
            field=models.ForeignKey(blank=True, help_text='The Attribute that is setting the start date for this task.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='domain.AttributeEntity', verbose_name='Start date attribute'),
        ),
    ]
